package com.jiaoyinbrother.monkeyking.activity

import android.annotation.SuppressLint
import android.app.Activity
import android.content.Intent
import android.text.TextUtils
import android.view.View
import com.camnter.easyrecyclerview.holder.EasyRecyclerViewHolder
import com.jiaoyinbrother.monkeyking.CarEntity
import com.jiaoyinbrother.monkeyking.Constants
import com.jiaoyinbrother.monkeyking.R
import com.jiaoyinbrother.monkeyking.adapter.PayMethodAdapter
import com.jiaoyinbrother.monkeyking.bean.OrderPayResult
import com.jiaoyinbrother.monkeyking.util.SharedPreferencesUtil
import com.jiaoyinbrother.monkeyking.view.IOSdialog.IOSAlertDialog
import com.jybrother.sineo.library.base.BaseActivity
import com.jybrother.sineo.library.bean.*
import com.jybrother.sineo.library.presenter.AccountPresenterImpl
import com.jybrother.sineo.library.presenter.OrderPresenterImpl
import com.jybrother.sineo.library.presenter.PayMoneyPresenterImpl
import com.jybrother.sineo.library.util.*
import com.jybrother.sineo.library.view.GeneralViewInterface
import com.jybrother.sineo.library.widget.GeneralBottomButton
import kotlinx.android.synthetic.main.activity_cashier_desk.*


/**
 * 支付收银台
 * by Soda on 2018/4/17.
 */
@SuppressLint("SetTextI18n")
class CashierDeskActivity : BaseActivity(), EasyRecyclerViewHolder.OnItemClickListener {

    companion object {
        const val PAY_TYPE_VEHICLE = 1
        const val PAY_TYPE_ILLEGAL = 2
        const val PAY_TYPE_RENT = 3
        const val PAY_TYPE_SETTLEMENT = 4
        const val PAY_TYPE_WEB = 5
    }

    private var mAdapter: PayMethodAdapter? = null

    private var mOrderInfo: OrderDetailResult? = null
    private var fromTakingPic: String? = null
    private var spUtil: SharedPreferencesUtil? = null
    // web pay
    private var isWebPay = false
    private var mWebPayBean: WebPayBean? = null
    //
    private var mBalanceAmount = 0
    private var mPaySettlementAmount: Int? = null
    private var mPaySettlementPayMethodList: ArrayList<PayMethodAvaiBean>? = null
    //
    private var mPayItem = ""// 支付项目类别
    private var mPayTn: String? = null
    private var mPayUrl: String? = null
    private var mPayType: Int? = null
    private var mPayMethod: Int? = null
    private var mPayAmount = 0

    private var disablePayChannel: String? = null
    private var enablePayChannel: String? = null
    private var isPaying = false

    @Suppress("UNCHECKED_CAST")
    private fun getExtra() {
        val bundle = intent?.extras
        mPayItem = bundle?.getString(ConstantsUtil.BUNDLE_EXTRA_KEY).toString()
        mOrderInfo = bundle?.getSerializable(ConstantsUtil.ORDER_RESULT) as OrderDetailResult
        fromTakingPic = bundle.getString("where are you from") //TODO 智能钥匙拍照过来的标识
        when (mPayItem) {
            ConstantsUtil.PAY_PROJECT_WEB -> mWebPayBean = bundle.getSerializable(ConstantsUtil.PAY_PROJECT_WEB) as WebPayBean
            ConstantsUtil.PAY_PROJECT_SETTLEMENT -> {
                mPaySettlementAmount = bundle.getInt("MorePay")
                mPaySettlementPayMethodList = bundle.getSerializable("SETTLEMENT_PAY_METHOD") as ArrayList<PayMethodAvaiBean>?
            }
        }
    }

    override fun getLayoutId(): Int {
        getExtra()
        return R.layout.activity_cashier_desk
    }

    override fun initView() {}

    override fun initToolBar() {
        mainTitle.text = "支付收银台"
    }

    override fun initListeners() {
        mAdapter?.setOnItemClickListener(this)
        onPay.setListener(object : GeneralBottomButton.SubmitListener {
            override fun submit() {
                if (mPayType == PAY_TYPE_SETTLEMENT) rechargePay() else orderPay()
            }
        })
    }

    override fun initData() {
        spUtil = SharedPreferencesUtil.getInstance()
        mAdapter = PayMethodAdapter(this@CashierDeskActivity)
        // 是否为Web支付
        isWebPay = (mPayType == PAY_TYPE_WEB)// 是否为web支付
        LogUtil.e("OrderPay initData : isWebPay-----> $isWebPay")
        // 支付项目
        mPayType = getPayType(mPayItem)
        LogUtil.e("OrderPay initData : mPayType-----> $mPayType")
        // 支付金额
        mPayAmount = getAmount(mPayType)
        LogUtil.e("OrderPay initData : mPayAmount-----> $mPayAmount")
        // 初始化支付方式,优先级：后台勾选->用户记忆->默认勾选第一项
        mPayMethod = getPayMethod()// 初始化支付方式
        LogUtil.e("OrderPay initData : mPayMethod-----> $mPayMethod")
        getAccountDetail()
    }

    override fun onItemClick(viewHolder: View?, position: Int) {
        val payMethod = mAdapter?.getItem<PayMethodAvaiBean>(position)
        if (payMethod?.id == PayMethodAvaiBean.ACCOUNTPAY && (mBalanceAmount < mPayAmount))
            return
        mAdapter?.selectPayMethod(payMethod?.id ?: -1)
        mPayMethod = payMethod?.id ?: 0
        LogUtil.e("当前点击:${payMethod?.name},支付方式:${payMethod?.id}")

        when (payMethod?.id) {
            PayMethodAvaiBean.WECHATPAY -> StatisticsUtil.onEvent(this, ConstantsUtil.CASHIER_DESK_CHOOSE_WXPAY, "")
            PayMethodAvaiBean.ALIPAY -> StatisticsUtil.onEvent(this, ConstantsUtil.CASHIER_DESK_CHOOSE_ALIPAY, "")
            PayMethodAvaiBean.YIBAOPAY -> StatisticsUtil.onEvent(this, ConstantsUtil.CASHIER_DESK_CHOOSE_BANKCARDPAY, "")
            PayMethodAvaiBean.ACCOUNTPAY -> StatisticsUtil.onEvent(this, ConstantsUtil.CASHIER_DESK_CHOOSE_ACCOUNT_PAY, "")
        }
    }

    /**
     * 获取支付类别：预付租金，车辆押金，违章押金，费用结算，web支付 order/pay
     * */
    private fun getPayType(payItem: String?): Int? {
        return when (payItem) {
        //  租金
            ConstantsUtil.PAY_PROJECT_RENT -> PAY_TYPE_RENT
        //  车辆押金
            ConstantsUtil.PAY_PROJECT_VEHICLE -> PAY_TYPE_VEHICLE
        //  违章押金
            ConstantsUtil.PAY_PROJECT_ILLEGAL -> PAY_TYPE_ILLEGAL
        //  确认结算
            ConstantsUtil.PAY_PROJECT_SETTLEMENT -> PAY_TYPE_SETTLEMENT
        //  H5支付
            else -> PAY_TYPE_WEB
        }
    }

    /**
     * 获取支付金额 order/pay
     * */
    private fun getAmount(mType: Int?): Int {
        return when (mType) {
            PAY_TYPE_RENT -> Util.getDepositBean(mOrderInfo, ConstantsUtil.PAY_PROJECT_RENT).amount.toInt()
            PAY_TYPE_VEHICLE -> Util.getDepositBean(mOrderInfo, ConstantsUtil.PAY_PROJECT_VEHICLE).amount.toInt()
            PAY_TYPE_ILLEGAL -> Util.getDepositBean(mOrderInfo, ConstantsUtil.PAY_PROJECT_ILLEGAL).amount.toInt()
            PAY_TYPE_SETTLEMENT -> mPaySettlementAmount ?: 0
            else -> mWebPayBean?.params?.amount ?: 0
        }
    }

    /**
     * 支付项目名称: 费用结算 & H5的支付内容，都显示为"应付金额"
     * */
    private fun getProject(mType: Int?): String {
        return when (mType) {
            PAY_TYPE_RENT -> "预付租金"
            PAY_TYPE_VEHICLE -> "车辆押金"
            PAY_TYPE_ILLEGAL -> "违章押金"
            else -> "应付金额"
        }
    }

    /**
     * 押金提示：车辆押金 & 违章押金
     * */
    private fun getDepositPrompt(mType: Int?): String {
        return when (mType) {
            PAY_TYPE_VEHICLE -> resources.getString(R.string.car_deposit_describe)
            PAY_TYPE_ILLEGAL -> resources.getString(R.string.car_deposit_illegal_describe)
            else -> ""
        }
    }

    /**
     * 可用支付列表
     * */
    private fun getPayMethodList(mType: Int?): ArrayList<PayMethodAvaiBean> {
        return when (mType) {
            PAY_TYPE_RENT -> getAvaiPayMethod(Util.getDepositBean(mOrderInfo, ConstantsUtil.PAY_PROJECT_RENT).pay_method_avai
                    ?: ArrayList())
            PAY_TYPE_VEHICLE -> getAvaiPayMethod(Util.getDepositBean(mOrderInfo, ConstantsUtil.PAY_PROJECT_VEHICLE).pay_method_avai
                    ?: ArrayList())
            PAY_TYPE_ILLEGAL -> getAvaiPayMethod(Util.getDepositBean(mOrderInfo, ConstantsUtil.PAY_PROJECT_ILLEGAL).pay_method_avai
                    ?: ArrayList())
            PAY_TYPE_SETTLEMENT -> getAvaiPayMethod(mPaySettlementPayMethodList ?: ArrayList())
            else -> defaultPayMethodList(mType)
        }
    }

    private fun getAvaiPayMethod(methodList: ArrayList<PayMethodAvaiBean>): ArrayList<PayMethodAvaiBean> {
        val avai = ArrayList<PayMethodAvaiBean>()
        for (method: PayMethodAvaiBean in methodList) {
            if (method.id == PayMethodAvaiBean.WECHATPAY || method.id == PayMethodAvaiBean.ALIPAY ||
                    method.id == PayMethodAvaiBean.YIBAOPAY || method.id == PayMethodAvaiBean.ACCOUNTPAY ||
                    method.id == PayMethodAvaiBean.UNIONPAY) {
                avai.add(method)
            }
        }
        return avai
    }

    /**
     * 初始化默认可用支付列表
     * */
    private fun defaultPayMethodList(mType: Int?): ArrayList<PayMethodAvaiBean> {
        val payMethodList = ArrayList<PayMethodAvaiBean>()
        payMethodList.add(PayMethodAvaiBean(PayMethodAvaiBean.WECHATPAY, "微信支付", "", 1))
        payMethodList.add(PayMethodAvaiBean(PayMethodAvaiBean.ALIPAY, "支付宝支付", "", 0))
        payMethodList.add(PayMethodAvaiBean(PayMethodAvaiBean.YIBAOPAY, "银行卡支付", "", 0))
        if (mType != PAY_TYPE_WEB) payMethodList.add(PayMethodAvaiBean(PayMethodAvaiBean.ACCOUNTPAY, "账户余额支付", "", 0))
        return payMethodList
    }

    /**
     * 初始化默认支付渠道(UNIONPAY:0，WECHATPAY:6，ALIPAY:7，ACCOUNTPAY:8,YIBAOPAY:12)
     * */
    private fun getPayMethod(): Int {
        val payMethodList = getPayMethodList(mPayType)
        // 存在用户习惯， 并且支付方式包含用户习惯
        for (payMethod: PayMethodAvaiBean in payMethodList) {
            if (spUtil?.payments != -1 && payMethod.id == spUtil?.payments) {// 选中
                LogUtil.e("getPayMethod ------> user finish")
                if (payMethod.id == PayMethodAvaiBean.ACCOUNTPAY && mBalanceAmount < mPayAmount) {
                    break
                }
                return payMethod.id
            }
        }
        // 后台提供勾选项
        for (payMethod: PayMethodAvaiBean in payMethodList) {
            if (payMethod.check == 1) {// 选中
                LogUtil.e("getPayMethod ------> system finish")
                return payMethod.id
            }
        }
        // 默认勾选第一项
        LogUtil.e("getPayMethod ------> defualt finish")
        return payMethodList[0].id
    }

    /**
     * 初始化支付收银台页面
     * */
    private fun setPayInfo(mType: Int?, result: OrderDetailResult?) {
        // 预付租金提示
        payPrompt.visibility = if (mType == PAY_TYPE_RENT) View.VISIBLE else View.GONE
        payPrompt.text = "请于${DateTimeUtil.getTimeAfter45min(result?.accept_time)}前完成支付"
        // 支付金额
        payAmount.text = TextUtil.setForegroundSize("¥$mPayAmount", 0.5f, 0, 1)
        // 支付项目
        payProject.text = getProject(mType)
        // 押金提示
        promptDepositLayout.visibility = if (mType == PAY_TYPE_VEHICLE || mType == PAY_TYPE_ILLEGAL) View.VISIBLE else View.GONE
        promptDeposit.text = getDepositPrompt(mType)
        // 支付方式
        mAdapter?.setAmout(mBalanceAmount, mPayAmount)
        mAdapter?.list = getPayMethodList(mType)
        LogUtil.e("PayMethodList =====> ${getPayMethodList(mType)}")
        payMethodRv.adapter = mAdapter
        mAdapter?.selectPayMethod(mPayMethod ?: -1)
    }

    override fun onBackPressed() {
        IOSAlertDialog(this@CashierDeskActivity).builder().setMsg("您还未支付哦，确定要离开吗？", true)
                .setCancelable(false).setPositiveButton("再想想") { }.setNegativeButton("去意已决") {
                    if (mPayType == PAY_TYPE_WEB) setResult(Constants.WEB_PAY_RESULT, Intent().putExtra("WEB_RESULT", "CANCEL"))
                    super.onBackPressed()
                }.show()
    }

    /**
     * 获取账户余额
     * */
    private fun getAccountDetail() {
        showLoadingDialog()
        val presenter = AccountPresenterImpl(this@CashierDeskActivity, AccountResult::class.java)
        presenter.addListener(accountModel)
        val request = UserDetailRequestBean()
        request.uid = SharedPreferencesUtil.getInstance().uid
        presenter.getAccount(request)
    }

    private val accountModel = object : GeneralViewInterface {
        override fun getCodeBeanFail(errorCode: Int) {
            mBalanceAmount = 0
            setPayInfo(mPayType, mOrderInfo)
            dismissLoadingDialog()
        }

        override fun getCodeObjectSuccess(bean: Any?) {
            val result = bean as AccountResult
            mBalanceAmount = if (result.code == CarEntity.SUCC) result.available_amount.toInt() else 0
            setPayInfo(mPayType, mOrderInfo)
            dismissLoadingDialog()
        }
    }

    /**
     * order/pay request
     * */
    private fun orderPay() {
        val presenter = OrderPresenterImpl(this@CashierDeskActivity, OrderPayResult::class.java)
        presenter.addListener(orderPayModel)
        val request = if (isWebPay) webPayRequest() else orderPayRequest()
        presenter.getOrderPay(request, mWebPayBean?.pay_path)
    }

    private fun orderPayRequest(): OrderPayRequest {
        val request = OrderPayRequest()
        request.orderid = mOrderInfo?.orderid
        request.pay_type = mPayType ?: -1
        request.amount = (mPayAmount * 100).toFloat()
        request.pay_method = mPayMethod ?: -1
        return request
    }

    private fun webPayRequest(): OrderPayRequest {
        val request = OrderPayRequest()
        val params = mWebPayBean?.params
        if (params != null) {
            request.orderid = params.orderid
            request.trans_type = params.trade_type
            request.amount = (params.amount * 100).toFloat()
            request.pay_method = mPayMethod ?: -1
        }
        return request
    }

    private val orderPayModel = object : GeneralViewInterface {
        override fun getCodeBeanFail(errorCode: Int) {
            showErrorToast(errorCode)
        }

        override fun getCodeObjectSuccess(bean: Any?) {
            val result = bean as OrderPayResult
            when (result.code) {
                CarEntity.SUCC -> {
                    spUtil?.savePayments(mPayMethod ?: -1)
                    mPayTn = result.tn
                    mPayUrl = result.url
                    disablePayChannel = result.disable_pay_channels
                    enablePayChannel = result.enable_pay_channels
                    callPayMethod(mPayUrl ?: "", mPayTn ?: "", mPayAmount, mPayMethod ?: 0)
                }
                else -> showToast(result.msg)
            }
        }
    }

    private fun rechargePay() {
        if (mPayMethod == PayMethodAvaiBean.ACCOUNTPAY) {
            // 如果使用余额支付，直接跳过返回费用结算
            setAccountPayResult()
        } else {
            val presenter = PayMoneyPresenterImpl(this, RechargeResult::class.java)
            presenter.addListener(rechargeModel)
            val request = RechargeRequest()
            request.uid = SharedPreferencesUtil.getInstance().uid
            request.pay_method = mPayMethod ?: -1
            request.amount = mPayAmount * 100
            presenter.goRecharge(request)
        }
    }

    private val rechargeModel = object : GeneralViewInterface {
        override fun getCodeBeanFail(errorCode: Int) {
            showErrorToast(errorCode)
        }

        override fun getCodeObjectSuccess(bean: Any?) {
            dismissLoadingDialog()
            val result = bean as RechargeResult
            when (result.code) {
                CarEntity.SUCC -> {
                    mPayTn = result.tn
                    mPayUrl = result.url
                    disablePayChannel = result.disable_pay_channels
                    enablePayChannel = result.enable_pay_channels
                    if (TextUtils.equals(result.tn, "0")) {
                        setAccountPayResult()
                    } else {
                        callPayMethod(mPayUrl ?: "", mPayTn ?: "", mPayAmount, mPayMethod ?: 0)
                    }
                }
                else -> showToast(result.msg)
            }
        }
    }

    private fun callPayMethod(url: String, tn: String, amount: Int, method: Int) {
        isPaying = true
        val payMethodBean = PayInfoBean(PayInfoBean.PAYMENT_TYPE_ZUCHE, url, tn, amount, method)
        val intent = Intent(this@CashierDeskActivity, PayMethodActivity::class.java)
        intent.putExtra(ConstantsUtil.START_PAYMETHOD, payMethodBean)
        intent.putExtra("disable_pay_channels", disablePayChannel)
        intent.putExtra("enable_pay_channels", enablePayChannel)
        startActivityForResult(intent, 101)
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == 101 && resultCode == Activity.RESULT_OK) {
            isPaying = false
            val code = data?.extras?.getInt(PayMethodActivity.RESULT_CODE)
            val message = data?.extras?.getString(PayMethodActivity.RESULT_MSG)
            val payInfo = PayInfoBean(PayInfoBean.PAYMENT_TYPE_ZUCHE, mPayUrl, mPayTn, mPayAmount, mPayMethod)
            // 当确认结算并且支付成功返回 确认结算进行结算，否则跳转支付状态页面
            if (mPayType == PAY_TYPE_SETTLEMENT && code == 1) {
                setAccountPayResult()
            } else {
                // 其他支付
                val intent = Intent(this@CashierDeskActivity, OrderPayResultActivity::class.java)
                intent.putExtra("PayResultCode", code)
                if (mPayType != PAY_TYPE_WEB) intent.putExtra(ConstantsUtil.ORDER_RESULT, mOrderInfo)
                if (TextUtils.equals(fromTakingPic, "TakingPic")) intent.putExtra("where are you from", "TakingPic")
                intent.putExtra("PayType", mPayType)
                intent.putExtra("PayResultCode", code)
                intent.putExtra("PayInfo", payInfo)
                startActivityForResult(intent, ConstantsUtil.PAYMENT_RESULT)
                finish()
            }

        }
    }

    /**
     * 账户余额支付费用结算成功
     * */
    private fun setAccountPayResult() {
        val intent = Intent()
        //                if (TextUtils.equals(mAccountPay, "SUCC"))
//                    intent.putExtra("mAccountPay", "SUCC")
        intent.putExtra("PayType", mPayType)
        intent.putExtra("PayMethod", mPayMethod)
        intent.putExtra("Tn", mPayTn ?: "0")
        setResult(CarEntity.ORDER_AUTO_PAY, intent)
        finish()
    }
}