package com.jiaoyinbrother.monkeyking.activity

import android.annotation.SuppressLint
import android.content.Intent
import android.os.Bundle
import android.text.TextUtils
import android.view.View
import com.jiaoyinbrother.monkeyking.CarEntity
import com.jiaoyinbrother.monkeyking.R
import com.jiaoyinbrother.monkeyking.bean.CouponBean
import com.jiaoyinbrother.monkeyking.fragment.CouponFragment
import com.jiaoyinbrother.monkeyking.fragment.CouponFragment.*
import com.jiaoyinbrother.monkeyking.network.AppSetting
import com.jybrother.sineo.library.adapter.MyFragmentAdapter
import com.jybrother.sineo.library.base.BaseActivity
import com.jybrother.sineo.library.bean.WebViewConfigEntity
import com.jybrother.sineo.library.util.ConstantsUtil
import com.jybrother.sineo.library.util.LogUtil
import com.jybrother.sineo.library.util.UrlUtil.COUPON_URL
import kotlinx.android.synthetic.main.activity_coupons.*
import java.util.ArrayList

/**
 * by Soda on 2018/2/26.
 */
class CouponsManageActivity : BaseActivity(), View.OnClickListener {

    companion object {
        val TAG = "CouponsManageActivity"
        val COUPON_MANAGE = 3
        val COUPON_SELECT = 1
    }

    private var unusedFragment: CouponFragment? = null
    private var usedFragment: CouponFragment? = null
    private var overdueFragment: CouponFragment? = null
    private var chooseCouponFragment: CouponFragment? = null

    private var mFlag: String? = null
    //    private var orderId: String? = null
    private var usableCoupons: ArrayList<CouponBean>? = null
    private var defSelectedCoupon: ArrayList<String>? = null
    private var selectedCoupon = ArrayList<CouponBean>()

    override fun getLayoutId(): Int {
        getExtra()
        return R.layout.activity_coupons
    }

    override fun initView() {
        couponsViewPager.offscreenPageLimit = if (isManagePages(mFlag)) COUPON_MANAGE else COUPON_SELECT
        val adapter = MyFragmentAdapter(supportFragmentManager)
        couponsViewPager.adapter = if (isManagePages(mFlag)) initManage(adapter) else initChoose(adapter)
    }

    override fun initToolBar() {
        LogUtil.e("$TAG initToolBar ......")
        mainTitle.text = if (isManagePages(mFlag)) "优惠券" else "使用优惠券"
        if (!isManagePages(mFlag)) subTitle.text = "使用说明"

    }

    override fun initListeners() {
        if (!isManagePages(mFlag)) {
            subTitle.setOnClickListener(this)
        }
        unusedCoupon.setOnClickListener(this)
        usedCoupon.setOnClickListener(this)
        overdueCoupon.setOnClickListener(this)
        unusedFragment?.setCouponCount(couponCountResult)
        chooseCouponFragment?.setChooseListener(chooseResult)
    }

    override fun initData() {
        subTitle.visibility = if (isManagePages(mFlag)) View.GONE else View.VISIBLE
        radioGroup.visibility = if (isManagePages(mFlag)) View.VISIBLE else View.GONE
    }

    override fun onClick(v: View?) {
        when (v?.id) {
            R.id.unusedCoupon -> couponsViewPager.currentItem = 0
            R.id.usedCoupon -> couponsViewPager.currentItem = 1
            R.id.overdueCoupon -> couponsViewPager.currentItem = 2
            R.id.subTitle -> useRules()
        }
    }

    private fun isManagePages(flag: String?): Boolean {
        return TextUtils.isEmpty(flag)
    }

    @Suppress("UNCHECKED_CAST")
    private fun getExtra() {
        LogUtil.e("$TAG getExtra")
        mFlag = intent.getStringExtra(ConstantsUtil.BUNDLE_EXTRA_KEY)
        // 预订
        usableCoupons = intent.getSerializableExtra("CouponsList") as ArrayList<CouponBean>?
        LogUtil.e("usableCoupons：$usableCoupons")
        defSelectedCoupon = intent.getSerializableExtra("defSelectedCoupon") as ArrayList<String>?
        LogUtil.e("defSelectedCoupon：$defSelectedCoupon")
    }

    private fun initManage(adapter: MyFragmentAdapter): MyFragmentAdapter {
        LogUtil.e("$TAG initManage")
        // 未使用
        unusedFragment = CouponFragment()
        val unused = Bundle()
        unused.putInt("STATUS_TYPE", CouponFragment.TYPE_UNUSED)
        unusedFragment?.arguments = unused
        // 已使用
        usedFragment = CouponFragment()
        val used = Bundle()
        used.putInt("STATUS_TYPE", CouponFragment.TYPE_USED)
        usedFragment?.arguments = used
        // 已过期
        overdueFragment = CouponFragment()
        val overdue = Bundle()
        overdue.putInt("STATUS_TYPE", CouponFragment.TYPE_OVERDUE)
        overdueFragment?.arguments = overdue

        adapter.addFragment(unusedFragment)
        adapter.addFragment(usedFragment)
        adapter.addFragment(overdueFragment)
        return adapter
    }

    private fun initChoose(adapter: MyFragmentAdapter): MyFragmentAdapter {
        LogUtil.e("$TAG initChoose")
        chooseCouponFragment = CouponFragment()
        val choose = Bundle()
        choose.putInt("STATUS_TYPE", CouponFragment.TYPE_CHOOSE)
        if (usableCoupons != null)
            choose.putSerializable("usableCoupons", usableCoupons)
        if (defSelectedCoupon != null)
            choose.putSerializable("defSelectedCoupon", defSelectedCoupon)
        chooseCouponFragment?.arguments = choose

        adapter.addFragment(chooseCouponFragment)
        return adapter
    }

    /**
     * use the rules H5
     * */
    private fun useRules() {
        val webViewConfigEntity = WebViewConfigEntity()
        webViewConfigEntity.title = "使用说明"
        webViewConfigEntity.url = AppSetting.getText_url() + COUPON_URL
        webViewConfigEntity.isNeedProgressBar = true
        webViewConfigEntity.isNeedShareButton = false
        webViewConfigEntity.imageUrl = ""
        webViewConfigEntity.description = ""

        val intent = Intent(this, GeneralWebViewActivity::class.java)
        intent.putExtra(CarEntity.FEATURE_ITEM, webViewConfigEntity)
        startActivity(intent)
    }

    private val couponCountResult = object : CouponCountListener {
        @SuppressLint("SetTextI18n")
        override fun onCountResult(unused: Int, used: Int, overdue: Int) {
            unusedCoupon.text = "未使用($unused)"
            usedCoupon.text = "已使用($used)"
            overdueCoupon.text = "已过期($overdue)"
        }
    }

    private val chooseResult = object : ChooseListener {
        override fun onChooseCancel() {
            selectedCoupon.clear()
            setCouponResult(selectedCoupon)
        }

        override fun onChooseResult(result: CouponBean) {
            selectedCoupon.clear()
            selectedCoupon.add(result)
            setCouponResult(selectedCoupon)
        }
    }

    private fun setCouponResult(selected: ArrayList<CouponBean>) {
        val intent = Intent()
        intent.putExtra("selectedCoupon", selected)
        setResult(ConstantsUtil.CHOOSE_COUPONS, intent)
        finish()
    }
}