package com.jiaoyinbrother.monkeyking.activity

import android.text.Editable
import android.text.TextUtils
import android.text.TextWatcher
import android.view.KeyEvent
import android.view.View
import android.widget.Button
import com.jiaoyinbrother.monkeyking.CarEntity
import com.jiaoyinbrother.monkeyking.R
import com.jiaoyinbrother.monkeyking.bean.ProjectBaseResult
import com.jiaoyinbrother.monkeyking.util.SharedPreferencesUtil
import com.jybrother.sineo.library.base.BaseActivity
import com.jybrother.sineo.library.bean.UserFeedbackRequest
import com.jybrother.sineo.library.presenter.UserFeedbackPresenterImpl
import com.jybrother.sineo.library.util.LogUtil
import com.jybrother.sineo.library.util.OldToastUtil
import com.jybrother.sineo.library.util.TextUtil
import com.jybrother.sineo.library.view.GeneralViewInterface
import kotlinx.android.synthetic.main.act_feedback.*

class FeedbackActivity : BaseActivity() {
    private var isSubmitTaskRun = false

    private val contentWatcher = object : TextWatcher {

        override fun onTextChanged(s: CharSequence, start: Int, before: Int, count: Int) {
            val editable = content_edittext.text.toString()
            val str = TextUtil.StringFilter(editable)
            if (editable != str) {
                content_edittext.setText(str)
                content_edittext.setSelection(str.length) // 光标置后
            }

        }

        override fun beforeTextChanged(s: CharSequence, start: Int, count: Int, after: Int) {

        }

        override fun afterTextChanged(s: Editable) {
            val lenth = s.length
            count_textview.setText("还可输入" + (200 - lenth) + "字")
            updateSubmitButton(s.toString())
        }
    }


    private val feedbackImpl = object : GeneralViewInterface {


        override fun getCodeBeanFail(errorCode: Int) {
            LogUtil.e("getCodeBeanFail" + errorCode)
            isSubmitTaskRun = false;
            dismissLoadingDialog()
            OldToastUtil.show(this@FeedbackActivity, "提交失败")
        }

        override fun getCodeObjectSuccess(bean: Any) {

            isSubmitTaskRun = false;
            dismissLoadingDialog()

            val baseResult = bean as ProjectBaseResult
            LogUtil.e("orderDetailResult=" + baseResult.toString())

            if (baseResult == null) {

                OldToastUtil.show(this@FeedbackActivity, "提交失败")
            } else {
                if (baseResult.code == CarEntity.SUCC) {
                    OldToastUtil.show(this@FeedbackActivity, "反馈成功")
                    finish()
                } else {
                    OldToastUtil.showerror(this@FeedbackActivity, baseResult.errCode)
                }

            }
        }
    }

    private fun submit() {
        if (!isSubmitTaskRun) {
            isSubmitTaskRun = true

            showLoadingDialog()

            val userFeedbackPresenter = UserFeedbackPresenterImpl(this@FeedbackActivity, ProjectBaseResult::class.java)
            userFeedbackPresenter.addListener(feedbackImpl)

            //                生成request
            val userFeedbackRequest = UserFeedbackRequest()
            if (null != content_edittext) {
                val contentStr = content_edittext.text.toString()
                userFeedbackRequest.feedback = contentStr
            }

            userFeedbackRequest.work_sheet_type = "PRODUCT_FEEDBACK"
            userFeedbackRequest.type = "OTHER"

            //			uid在租车就是手机号,自驾游不是
            if (!TextUtils.isEmpty(SharedPreferencesUtil.getInstance().uid)) {
                userFeedbackRequest.userphone = SharedPreferencesUtil.getInstance().uid
            }

            userFeedbackPresenter.submitFeedback(userFeedbackRequest)
        }
    }

    override fun getLayoutId(): Int {
        return R.layout.act_feedback
    }

    override fun initView() {

    }

    override fun initToolBar() {

    }

    override fun initListeners() {
        content_edittext.addTextChangedListener(contentWatcher)

    }

    override fun initData() {
        (findViewById(R.id.ivTitleName) as Button).text = resources.getString(R.string.feedback)
        findViewById(R.id.ivTitleBtnRight_ll).visibility = View.INVISIBLE

        updateSubmitButton("")
    }


    private fun updateSubmitButton(editable: String) {
        bottom_button.isEnabled = !editable.isEmpty()

    }

    fun onPost(view: View) {
        if (null == content_edittext)
            return
        val str2 = content_edittext.text.toString()
        if (null == str2 || str2.isEmpty()) {
            OldToastUtil.show(this@FeedbackActivity, "请输入内容")
            return
        }
        submit()
    }

    override fun onKeyDown(keyCode: Int, event: KeyEvent): Boolean {
        if (keyCode == KeyEvent.KEYCODE_BACK) {
            finish()
        }
        return false
    }

}
