package com.jiaoyinbrother.monkeyking.activity

import android.content.Intent
import android.text.TextUtils
import android.view.View
import com.camnter.easyrecyclerview.holder.EasyRecyclerViewHolder
import com.jiaoyinbrother.monkeyking.CarEntity
import com.jiaoyinbrother.monkeyking.Constants
import com.jiaoyinbrother.monkeyking.R
import com.jiaoyinbrother.monkeyking.adapter.OrderListAdapter
import com.jiaoyinbrother.monkeyking.fragment.OrderListFragment
import com.jiaoyinbrother.monkeyking.util.SharedPreferencesUtil
import com.jybrother.sineo.library.base.BaseActivity
import com.jybrother.sineo.library.bean.*
import com.jybrother.sineo.library.presenter.OrderPresenterImpl
import com.jybrother.sineo.library.util.ConstantsUtil
import com.jybrother.sineo.library.util.StatisticsUtil
import com.jybrother.sineo.library.view.GeneralViewInterface
import com.jybrother.sineo.library.widget.OrderListSortView
import kotlinx.android.synthetic.main.activity_manager_order_list.*
import java.util.*

class ManageOrderListActivity : BaseActivity(), View.OnClickListener, EasyRecyclerViewHolder.OnItemClickListener {

    companion object {
        const val PAGE_SIZE = 10
        private const val NET_ERROR = 1
        private const val NO_ORDERS = 2
    }

    private var carId: String? = null
    private var page = 1
    private var orderType = OrderListFragment.STATUS_MANAGER_HANDOVER
    private var sortFactor = OrderListSortView.DEFAULT
    private var sidx = ""// 排序字段(startTime, endTime)
    private var sord = ""// 排序方式(ASC,DESC)
    private var isFooterAdded = false
    private var isRenewalFlag = false
    private var mOrders: ArrayList<OrdersBean>? = null
    private var mAdapter: OrderListAdapter? = null

    override fun getLayoutId(): Int {
        carId = intent.getStringExtra("see_all_car_id")
        return R.layout.activity_manager_order_list
    }

    override fun initView() {
    }

    override fun initToolBar() {
        mainTitle.text = "订单管理"
        subImg1.setBackgroundResource(R.mipmap.ic_order_search)
    }

    override fun initListeners() {
        subImg1.setOnClickListener(this)
        mAdapter?.setOnItemClickListener(this)
        // 车管
        mgrHandoverRb.setOnClickListener(this)
        mgrGoingRb.setOnClickListener(this)
        mgrSettledRb.setOnClickListener(this)
        mgrCompletedRb.setOnClickListener(this)
        mgrCancelRb.setOnClickListener(this)
        // 排序
        orderSortView.setListener(object : OrderListSortView.OrderSortListener {
            override fun onDefault(sort: Int) {
                onSort(sort, false)
            }

            override fun onTakeCar(sort: Int, isASC: Boolean) {
                onSort(sort, isASC)
            }

            override fun onReturnCar(sort: Int, isASC: Boolean) {
                onSort(sort, isASC)
            }
        })
        refreshLayout.setRefreshListener {
            page = 1
            getOrders(orderType)
        }
        refreshLayout.setMoreListener {
            getOrders(orderType)
        }
        mAdapter?.onItemClick(object : OrderListAdapter.OrderItemClickListener {

            override fun onSmallOne(order: OrdersBean, position: Int) {
                isRenewalFlag = true
                getOrderDetail(order)
            }

            override fun onSmallTwo(order: OrdersBean, position: Int) {
                isRenewalFlag = false
                getOrderDetail(order)
            }

            override fun onBigThree(order: OrdersBean, position: Int) {
                bigThreeEvent(order)
            }
        })
        mAdapter?.setOnItemClickListener { p0, position ->
            val order = mAdapter?.getItem<OrdersBean>(position)
            if (!TextUtils.isEmpty(order?.orderid)) {
                val intent = Intent(this@ManageOrderListActivity, OrderDetailActivity::class.java)
                intent.putExtra(ConstantsUtil.ORDER_ID, order?.orderid)
                startActivityForResult(intent, Constants.ORDER_DETAIL_CODE)
            }
        }
    }

    override fun initData() {
        mOrders = ArrayList()
        refreshLayout.setFooter()
        mAdapter = OrderListAdapter(this@ManageOrderListActivity)
        orderlistRv.adapter = mAdapter
        getOrders(orderType)
    }
    override fun onClick(v: View?) {
        when (v?.id) {
        // 车管
            R.id.mgrHandoverRb -> orderType = OrderListFragment.STATUS_MANAGER_HANDOVER
            R.id.mgrGoingRb -> orderType = OrderListFragment.STATUS_MANAGER_GOING
            R.id.mgrSettledRb -> orderType = OrderListFragment.STATUS_MANAGER_SETTLED
            R.id.mgrCompletedRb -> orderType = OrderListFragment.STATUS_MANAGER_COMPLETE
            R.id.mgrCancelRb -> orderType = OrderListFragment.STATUS_MANAGER_CANCEL
            R.id.subImage1 -> startActivity(OrderSearchActivity::class.java)
        }
        if (v?.id != R.id.subImage1) {
            page = 1
            getOrders(orderType)
        }
    }

    override fun onItemClick(p0: View?, p1: Int) {
    }

    private fun getOrderStatusList(type: Int): ArrayList<String> {
        return when (type) {
        // 车管
            OrderListFragment.STATUS_MANAGER_HANDOVER -> ArrayList(Arrays.asList("ORDER_ACCEPTED", "ORDER_CAR_PAYED", "ORDER_ILLEGAL_PAYED"))
            OrderListFragment.STATUS_MANAGER_GOING -> ArrayList(Arrays.asList("ORDER_EXECUTING"))
            OrderListFragment.STATUS_MANAGER_SETTLED -> ArrayList(Arrays.asList("ORDER_BILL_APPLIED"))
            OrderListFragment.STATUS_MANAGER_COMPLETE -> ArrayList(Arrays.asList("ORDER_BILL_ACCEPTED", "ORDER_FINISTHED"))
            else -> ArrayList(Arrays.asList("ORDER_CANCELED"))
        }
    }

    private fun getOrders(type: Int) {
        val presenter = OrderPresenterImpl(this@ManageOrderListActivity, OrdersResult::class.java)
        presenter.addListener(mOrdersModel)
        val request = OrdersRequest()
        request.uid = SharedPreferencesUtil.getInstance().uid
        request.page = page
        request.page_size = PAGE_SIZE
        request.carid = carId
        request.status = 4
        request.status_list = getOrderStatusList(type)
        if (sortFactor != OrderListSortView.DEFAULT) {
            request.sidx = sidx
            request.sord = sord
        }
        presenter.getOrderList(request)
    }

    private val mOrdersModel = object : GeneralViewInterface {
        override fun getCodeBeanFail(errorCode: Int) {
            refreshLayout.refreshIsEnd()
            refreshLayout.loadMoreIsEnd()
            emptyLayout(true, NET_ERROR)
        }

        override fun getCodeObjectSuccess(bean: Any?) {
            refreshLayout.refreshIsEnd()
            refreshLayout.loadMoreIsEnd()
            val result = bean as OrdersResult
            when (result.code) {
                CarEntity.SUCC -> {
                    // 第一页，清除数据，footer
                    if (page == 1) {
                        mOrders?.clear()
                        deleteFooter()
                    }
                    mOrders?.addAll(result.orders ?: ArrayList())
                    if (result.orders?.size ?: 0 < PAGE_SIZE) {
                        // 最后的数据
                        if (result.orders?.size ?: 0 > 0) {
                            addFooter()
                        } else {
                            // 个数为0， 如果是第一页，空页面；不是第一页，添加footer
                            if (page > 1) addFooter()
                        }
                    }
                    mAdapter?.list = mOrders
                    mAdapter?.notifyDataSetChanged()
                    page++
                }
                else -> showToast(result.msg)
            }
            if (mAdapter?.itemCount == 0) emptyLayout(true, NO_ORDERS) else emptyLayout(false, NO_ORDERS)
        }
    }

    private fun deleteFooter() {
        mAdapter?.addFooter(false)
        isFooterAdded = false
    }

    private fun addFooter() {
        if (isFooterAdded)
            return
        mAdapter?.addFooter(true)
        val footer = OrdersBean()
        mOrders?.add(footer)
        isFooterAdded = true
    }

    private fun onSort(type: Int?, b: Boolean) {
        sortFactor = type ?: 0
        sord = if (b) OrderListSortView.ASC else OrderListSortView.DESC
        sidx = when (sortFactor) {
            OrderListSortView.TAKE_CAR -> OrderListSortView.START_TIME
            OrderListSortView.RETURN_CAR -> OrderListSortView.END_TIME
            else -> ""
        }
        page = 1
        getOrders(orderType)
    }

    private fun getOrderDetail(order: OrdersBean?) {
        val presenter = OrderPresenterImpl(this@ManageOrderListActivity, OrderDetailResult::class.java)
        presenter.addListener(orderDetailModel)
        val request = OrderDetailRequest()
        request.uid = SharedPreferencesUtil.getInstance().uid
        request.orderid = order?.orderid
        presenter.getOrderDetail(request)
    }

    private val orderDetailModel = object : GeneralViewInterface {
        override fun getCodeBeanFail(errorCode: Int) {
        }

        override fun getCodeObjectSuccess(bean: Any) {
            val result = bean as OrderDetailResult
            when (result.code) {
                CarEntity.SUCC -> {
                    handingOrderDetailEvent(result)
                }
                else -> showToast(result.msg)
            }
        }
    }

    private fun handingOrderDetailEvent(result: OrderDetailResult) {
        if (TextUtils.isEmpty(result.orderid)) {
            showToast("订单异常，请咨询客服处理")
            return
        }
        when (result.status) {
            OrdersBean.ORDER_STATUS_1 -> {// 已接受
                // 预付租金
                StatisticsUtil.onEvent(this@ManageOrderListActivity, ConstantsUtil.ORDER_LIST_PREPAY, ConstantsUtil.ORDER_LIST_PREPAY)
                onPayDeposit(result, ConstantsUtil.PAY_PROJECT_RENT)
            }
            OrdersBean.ORDER_STATUS_3 -> {// 已支付车辆押金
                onPayDeposit(result, ConstantsUtil.PAY_PROJECT_ILLEGAL)
            }
            OrdersBean.ORDER_STATUS_5 -> {// 用车中
                if (result.bill == null) {
                    showToast("订单异常，请咨询客服处理")
                    return
                }
                if (isRenewalFlag) {
                    // 续租
                    StatisticsUtil.onEvent(this@ManageOrderListActivity, ConstantsUtil.ORDER_LIST_RENEW, ConstantsUtil.ORDER_LIST_RENEW)
                    val intent = Intent(this@ManageOrderListActivity, ReletActivity::class.java)
                    intent.putExtra(ConstantsUtil.ORDER_RESULT, result)
                    startActivityForResult(intent, Constants.RELET_CODE)
                } else {
                    // 提前还车
                    StatisticsUtil.onEvent(this@ManageOrderListActivity, ConstantsUtil.ORDER_LIST_RETURN_CAR, ConstantsUtil.ORDER_LIST_RETURN_CAR)
                    val intent = Intent(this@ManageOrderListActivity, AdvanceBackActivity::class.java)
                    intent.putExtra(ConstantsUtil.ORDER_RESULT, result)
                    startActivityForResult(intent, Constants.ADVANCE_CODE)
                }
            }
            OrdersBean.ORDER_STATUS_6 -> {
                val intent = Intent(this@ManageOrderListActivity, SettlementForRenterActivity::class.java)
                intent.putExtra("Flag", "确认结算")
                intent.putExtra("order_detail", result)
                startActivityForResult(intent, Constants.CLOSE_ACCT_CODE)
            }
        }
    }

    private fun onPayDeposit(result: OrderDetailResult, mType: String) {
        val intent = Intent(this@ManageOrderListActivity, CashierDeskActivity::class.java)
        intent.putExtra(ConstantsUtil.BUNDLE_EXTRA_KEY, mType)
        intent.putExtra(ConstantsUtil.ORDER_RESULT, result)
        startActivityForResult(intent, ConstantsUtil.PAYMENT_RESULT)
    }

    private fun bigThreeEvent(order: OrdersBean?) {
        if (order?.status ?: 0 in 7..10) run {
            val intent = Intent(this@ManageOrderListActivity, EvaluateActivity::class.java)
            intent.putExtra(CarEntity.ORDER_OWNER_KEY, CarEntity.ORDER_RENTER)
            intent.putExtra(ConstantsUtil.ORDER_ID, order?.orderid)
            intent.putExtra(CarEntity.CMMT_SINGLE_ID, order?.ownerid)
            intent.putExtra("CAR_ID", order?.carid)
            startActivityForResult(intent, Constants.COMMENT_SINGLE_CODE)
        }
    }

    private fun emptyLayout(b: Boolean, i: Int) {
        refreshLayout.visibility = if (b) View.GONE else View.VISIBLE
        emptyView.visibility = if (b) View.VISIBLE else View.GONE
        when (i) {
            NET_ERROR -> {
                emptyView.setEmptyData(R.mipmap.ic_no_network, "", ConstantsUtil.NO_NETWORK, "")
                emptyView.setNegativeButton("重新加载", View.OnClickListener { getOrders(orderType) })
            }
            NO_ORDERS -> {
                emptyView.setEmptyData(R.mipmap.ic_no_orders, "", "您还没有订单哦~", "")
            }
        }
    }
}