package com.jiaoyinbrother.monkeyking.activity

import android.os.AsyncTask
import android.view.View
import android.widget.Button
import android.widget.TextView
import com.camnter.easyrecyclerview.adapter.EasyRecyclerViewAdapter
import com.camnter.easyrecyclerview.holder.EasyRecyclerViewHolder
import com.jiaoyinbrother.monkeyking.CarEntity
import com.jiaoyinbrother.monkeyking.R
import com.jiaoyinbrother.monkeyking.bean.GetJFRecordsEntity
import com.jiaoyinbrother.monkeyking.bean.GetJFRecordsResult
import com.jiaoyinbrother.monkeyking.bean.JFrecords
import com.jiaoyinbrother.monkeyking.network.CarLib
import com.jiaoyinbrother.monkeyking.util.NetUtil
import com.jiaoyinbrother.monkeyking.util.SharedPreferencesUtil
import com.jybrother.sineo.library.base.BaseActivity
import com.jybrother.sineo.library.util.LogUtil
import kotlinx.android.synthetic.main.act_my_integral.*

class MyIntergralActivity : BaseActivity() {

    private var pageNo = 1
    private var mCarLib: CarLib? = null
    private var isIntegralDetailTaskRun = false

    private var integralRecyclerAdapter: IntegralRecyclerAdapter? = null
    private var mData: ArrayList<JFrecords>? = null

    override fun getLayoutId(): Int {
        return R.layout.act_my_integral
    }

    override fun initView() {

    }

    override fun initToolBar() {

    }

    override fun initListeners() {
        refresh_layout.setRefreshListener {
            pageNo = 1
            getIntegralDetail()
        }
        refresh_layout.setMoreListener { getIntegralDetail() }
    }

    override fun initData() {
        (findViewById(R.id.ivTitleName) as Button).text = resources.getString(R.string.title_integral2)
        findViewById(R.id.ivTitleBtnRight_ll).visibility = View.INVISIBLE

        mData = ArrayList()
        pageNo = 1
        refresh_layout.setFooter()

        integralRecyclerAdapter = IntegralRecyclerAdapter()
        recyclerview_integral.adapter = integralRecyclerAdapter

        getIntegralDetail()
    }

    /**
     * 获取用户账户余额
     */
    private fun getIntegralDetail() {
        if (!isIntegralDetailTaskRun) {
            isIntegralDetailTaskRun = true
            IntegralDetailAsyncTask().execute()
        }
    }

    private inner class IntegralDetailAsyncTask : AsyncTask<Void, Void, GetJFRecordsResult>() {

        override fun onPreExecute() {
            super.onPreExecute()
            showLoadingDialog()
        }

        override fun doInBackground(vararg params: Void): GetJFRecordsResult {
            if (null == mCarLib)
                mCarLib = CarLib.getInstance(applicationContext)
            val gre = GetJFRecordsEntity()
            gre.uid = SharedPreferencesUtil.getInstance().uid
            gre.page = pageNo
            gre.page_size = PAGE_SIZE
            var grr = GetJFRecordsResult()
            try {
                grr = mCarLib!!.postRequest(gre.toJson(gre), "account/jf/get_records",
                        GetJFRecordsResult::class.java) as GetJFRecordsResult
            } catch (e: Exception) {
                NetUtil.handlePostException(grr, e)
            }

            return grr
        }

        override fun onPostExecute(result: GetJFRecordsResult?) {
            super.onPostExecute(result)
            dismissLoadingDialog()

            refresh_layout.refreshIsEnd()
            refresh_layout.loadMoreIsEnd()

            isIntegralDetailTaskRun = false

            if (result == null) return

            when (result.code) {
                CarEntity.SUCC -> {

//                    第一页，清除数据，footer
                    if (pageNo == 1) {
                        mData?.clear()
//                        添加header使用数据
                        var header = JFrecords()
                        header.point = result.total
                        mData?.add(header)
                        deleteFooter()
                    }

                    mData?.addAll(result.records ?: ArrayList())

                    if (result.records.size < PAGE_SIZE) {
//                        最后的数据
                        if (result.records.size > 0) {
                            addFooter()
                        } else {
//                            个数为0， 如果是第一页，空页面；不是第一页，添加footer
                            if (pageNo > 1) {
                                addFooter()
                            }
                        }
                    } else {
                        // 还有数据
                    }
                    integralRecyclerAdapter?.list = mData
                    integralRecyclerAdapter?.notifyDataSetChanged()
                    pageNo++

                }
                else -> showToast(result.msg)
            }
            LogUtil.e("integralRecyclerAdapter?.itemCount =" +integralRecyclerAdapter?.itemCount)
            if (integralRecyclerAdapter?.itemCount!! <= 1) {
                my_integral_no_data.visibility = View.VISIBLE
                refresh_layout.visibility = View.GONE
            } else {
                my_integral_no_data.visibility = View.GONE
                refresh_layout.visibility = View.VISIBLE
            }
        }
    }


    private fun deleteFooter() {
        integralRecyclerAdapter?.deleteFooter()
        isFooterAdded = false
    }

    private var isFooterAdded = false

    private fun addFooter() {

        if (isFooterAdded)
            return;

        integralRecyclerAdapter?.addFooter()
        val footer = JFrecords()// bug 重复添加
        mData?.add(footer)
        isFooterAdded = true
    }


    companion object {
        private val PAGE_SIZE = 20
    }
}

class IntegralRecyclerAdapter : EasyRecyclerViewAdapter() {
    private var footer = false

    override fun getRecycleViewItemType(position: Int): Int {
        return when (position) {
            0 -> 0
            itemCount - 1 -> {
                if (footer) 2 else 1
            }
            else -> 1
        }
    }

    override fun onBindRecycleViewHolder(viewHolder: EasyRecyclerViewHolder, position: Int) {
        val recycleViewItemType = getRecycleViewItemType(position)
        when (recycleViewItemType) {
            0 -> onBindHeaderViewHolder(viewHolder, position)
            1 -> onBindIntegralItemViewHolder(position, viewHolder)
        }
    }

    private fun onBindHeaderViewHolder(viewHolder: EasyRecyclerViewHolder, position: Int) {
        val entity = list[position] as JFrecords

        var total = viewHolder.findViewById<TextView>(R.id.all)
        total.text = entity.point.toString()

    }

    private fun onBindIntegralItemViewHolder(position: Int, viewHolder: EasyRecyclerViewHolder) {
        val entity = list[position] as JFrecords

        var type = viewHolder.findViewById<TextView>(R.id.type)
        var count = viewHolder.findViewById<TextView>(R.id.count)
        var time = viewHolder.findViewById<TextView>(R.id.time)
        if (null != entity) {
            if (null != entity.time) {
                val strs = entity.time.split("[ ]".toRegex()).dropLastWhile({ it.isEmpty() }).toTypedArray()
                time.text = strs[0]
            }
            count.text = entity.point.toString()
            type.text = entity.status_name

        }
    }

    override fun getItemLayouts(): IntArray {
        return intArrayOf(R.layout.layout_my_intergral_item_header, R.layout.adapter_wkcoinitem, R.layout.layout_footer_all_data)
    }


    fun addFooter() {
        this.footer = true
    }

    fun deleteFooter() {
        this.footer = false
    }

}
