package com.jiaoyinbrother.monkeyking.activity

import android.content.Intent
import android.text.TextUtils
import android.view.View
import com.camnter.easyrecyclerview.holder.EasyRecyclerViewHolder
import com.jiaoyinbrother.monkeyking.CarEntity
import com.jiaoyinbrother.monkeyking.Constants
import com.jiaoyinbrother.monkeyking.R
import com.jiaoyinbrother.monkeyking.adapter.OrderListAdapter
import com.jiaoyinbrother.monkeyking.util.NetUtil
import com.jiaoyinbrother.monkeyking.util.SharedPreferencesUtil
import com.jiaoyinbrother.monkeyking.util.Util
import com.jybrother.sineo.library.base.BaseActivity
import com.jybrother.sineo.library.bean.*
import com.jybrother.sineo.library.presenter.OrderPresenterImpl
import com.jybrother.sineo.library.util.ConstantsUtil
import com.jybrother.sineo.library.util.OldToastUtil
import com.jybrother.sineo.library.util.TextUtil
import com.jybrother.sineo.library.view.GeneralViewInterface
import com.jybrother.sineo.library.widget.GeneralBottomButton
import kotlinx.android.synthetic.main.activity_order_search.*

class OrderSearchActivity : BaseActivity() {

    private var page = 1
    private var mAdapter: OrderListAdapter? = null
    private var mOrders: ArrayList<OrdersBean>? = null
    override fun getLayoutId(): Int {
        return R.layout.activity_order_search
    }

    override fun initView() {
    }

    override fun initToolBar() {
        mainTitle.text = "订单查询"
    }

    override fun initListeners() {
        mAdapter?.setOnItemClickListener({ view, position ->
            val order = mAdapter?.getItem<OrdersBean>(position)
            if (!TextUtils.isEmpty(order?.orderid)) {
                val intent = Intent(this@OrderSearchActivity, OrderDetailActivity::class.java)
                intent.putExtra(ConstantsUtil.ORDER_ID, order?.orderid)
                startActivityForResult(intent, Constants.ORDER_DETAIL_CODE)
            }
        })
        onSearch.setListener(object : GeneralBottomButton.SubmitListener {
            override fun submit() {
                if (!NetUtil.isNetworkAvailable(this@OrderSearchActivity)) {
                    OldToastUtil.showerror(this@OrderSearchActivity, OldToastUtil.TIMEOUT_EXCEPTION)
                    return
                }
                if (TextUtils.isEmpty(searchInfo.text.toString())) {
                    showToast("请输入订单号/手机号")
                    return
                }
                if (searchInfo.text.toString().length < 8) {
                    showToast("请输入正确订单号/手机号，进行查询")
                    return
                }
                if (searchInfo.text.toString().length in 9..10) {
                    showToast("请输入正确手机号，进行查询")
                    return
                }
                val patter = TextUtil.phonePatter
                if (!TextUtil.matchers_input(patter, searchInfo.text.toString()) && searchInfo.text.toString().length == 11) {
                    showToast("手机号格式错误")
                    return
                }
                showLoadingDialog()
                if (searchInfo.text.toString().length == 11) searchPhone() else searchOrder()
            }
        })

        refreshLayout.setRefreshListener {
            page = 1
            searchPhone()
        }
        refreshLayout.setMoreListener {
            searchPhone()
        }
    }

    override fun initData() {
        mOrders = ArrayList()
        refreshLayout.setFooter()
        mAdapter = OrderListAdapter(this@OrderSearchActivity)
        resultRv.adapter = mAdapter
    }

    private fun searchPhone() {
        val presenter = OrderPresenterImpl(this@OrderSearchActivity, OrdersResult::class.java)
        presenter.addListener(mSearchPhoneModel)
        val request = OrdersRequest()
        request.uid = SharedPreferencesUtil.getInstance().uid
        request.renterid = searchInfo.text.toString()
        request.page = page
        request.page_size = 10
        request.status = 4
        presenter.getOrderList(request)
    }

    private val mSearchPhoneModel = object : GeneralViewInterface {
        override fun getCodeBeanFail(errorCode: Int) {
            refreshLayout.refreshIsEnd()
            refreshLayout.loadMoreIsEnd()
            showErrorToast(errorCode)
            dismissLoadingDialog()
        }

        override fun getCodeObjectSuccess(bean: Any?) {
            refreshLayout.refreshIsEnd()
            refreshLayout.loadMoreIsEnd()
            dismissLoadingDialog()
            val result = bean as OrdersResult
            when (result.code) {
                CarEntity.SUCC -> {
                    mOrders?.clear()
                    if (!Util.isListEmpty(result.orders)) {
                        showList(true)
                        mOrders?.addAll(result.orders ?: ArrayList())
                        page++
                    } else {
                        mOrders?.addAll(ArrayList())
                    }
                    mAdapter?.list = mOrders
                    mAdapter?.notifyDataSetChanged()
                    if (mAdapter?.itemCount == 0) emptyLayout(true) else emptyLayout(false)
                }
                else -> showToast(result.msg)
            }

        }
    }

    private fun searchOrder() {
        val presenter = OrderPresenterImpl(this@OrderSearchActivity, OrderDetailResult::class.java)
        presenter.addListener(mSearchOrderModel)
        val request = OrderDetailRequest()
        request.uid = SharedPreferencesUtil.getInstance().uid
        request.orderid = searchInfo.text.toString()
        presenter.getOrderDetail(request)
    }

    private val mSearchOrderModel = object : GeneralViewInterface {
        override fun getCodeBeanFail(errorCode: Int) {
            showErrorToast(errorCode)
            dismissLoadingDialog()
        }

        override fun getCodeObjectSuccess(bean: Any?) {
            dismissLoadingDialog()
            val result = bean as OrderDetailResult
            when (result.code) {
                CarEntity.SUCC -> {
                    if (!TextUtils.isEmpty(result.orderid)) {
                        val intent = Intent(this@OrderSearchActivity, OrderDetailActivity::class.java)
                        intent.putExtra(ConstantsUtil.ORDER_ID, result.orderid)
                        startActivity(intent)
                    } else {
                        emptyLayout(true)
                    }
                }
                else -> showToast(result.msg)
            }
        }
    }

    private fun showList(isList: Boolean) {
        if (isList) {
            orderSearchLayout.visibility = View.GONE
            refreshLayout.visibility = View.VISIBLE
        } else {
            orderSearchLayout.visibility = View.VISIBLE
            refreshLayout.visibility = View.GONE
            emptyView.visibility = View.GONE
        }
    }

    override fun onBackPressed() {
        if (orderSearchLayout.visibility == View.VISIBLE) {
            super.onBackPressed()
        } else {
            showList(false)
        }
    }

    private fun emptyLayout(b: Boolean) {
        refreshLayout.visibility = if (b) View.GONE else View.VISIBLE
        emptyView.visibility = if (b) View.VISIBLE else View.GONE
        emptyView.setEmptyData(R.mipmap.ic_no_coupons, "", "未查询到相关订单。请重新核对!", "")
    }
}