package com.jiaoyinbrother.monkeyking.activity

import android.app.Activity
import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.os.Handler
import com.jiaoyinbrother.monkeyking.CarEntity
import com.jiaoyinbrother.monkeyking.R
import com.jiaoyinbrother.monkeyking.util.SharedPreferencesUtil
import com.jiaoyinbrother.monkeyking.wxapi.WXPayEntryActivity
import com.jybrother.sineo.library.base.BaseActivity
import com.jybrother.sineo.library.bean.PayInfoBean
import com.jybrother.sineo.library.bean.PayQueryRequest
import com.jybrother.sineo.library.bean.PayQueryResult
import com.jybrother.sineo.library.bean.WebViewConfigEntity
import com.jybrother.sineo.library.presenter.PayMoneyPresenterImpl
import com.jybrother.sineo.library.util.ConstantsUtil
import com.jybrother.sineo.library.util.LogUtil
import com.jybrother.sineo.library.util.PayUtil
import com.jybrother.sineo.library.util.UrlUtil
import com.jybrother.sineo.library.view.GeneralViewInterface
import com.unionpay.UPPayAssistEx

class PayMethodActivity : BaseActivity() {

    private var mPayMethod = -1
    private var mPayAmount = 0
    private var mPayTn = ""
    private var mPayUrl = ""
    private var disablePayChannels: String? = null
    private var enablePayChannels: String? = null

    override fun getLayoutId(): Int {
        return R.layout.activity_paymethod
    }

    override fun initView() {
        disablePayChannels = intent.extras.getString("disable_pay_channels")
        enablePayChannels = intent.extras.getString("enable_pay_channels")

        val payMethodBean = intent.extras.getSerializable(ConstantsUtil.START_PAYMETHOD) as PayInfoBean
        mPayTn = payMethodBean.tn
        mPayMethod = payMethodBean.payMethod
        mPayAmount = payMethodBean.amount
        mPayUrl = payMethodBean.url
        val payUtil = PayUtil(this@PayMethodActivity)
        payUtil.setListener(object : PayUtil.PayUtilListener {

            override fun onUnionPay(payTn: String?) {
                try {
                    UPPayAssistEx.startPay(this@PayMethodActivity, null, null, payTn, UrlUtil.getUniPaymode())
                } catch (e: Exception) {
                    LogUtil.e("银联crash---------------------------------------------")
                }
            }

            override fun onYiBaoPay(request: WebViewConfigEntity) {
                val intent = Intent(this@PayMethodActivity, GeneralWebViewActivity::class.java)
                intent.putExtra("featureItem", request)
                startActivityForResult(intent, ConstantsUtil.YIBAO_PAY_RESULT)
            }

            override fun onAliPayResult(result: String) {
                getQueryResult()
            }

            override fun onAccountPay() {
                setPayResult(SUCC, "支付成功")
            }
        })
        payUtil.aLiPaymentLimit(disablePayChannels ?: "", enablePayChannels ?: "")
        payUtil.startPayment(this@PayMethodActivity, mPayTn, mPayMethod, mPayAmount, mPayUrl, payMethodBean.flag)
    }

    override fun initToolBar() {}

    override fun initListeners() {}

    override fun initData() {}

    override fun onStart() {
        super.onStart()
        showLoadingDialog()
        registerReceiver()
    }

    /**
     * 微信pay回调
     * */
    private fun registerReceiver() {
        val intentFilter = IntentFilter()
        intentFilter.addAction(WXPayEntryActivity.WECHAT_PAY_RESULTS)
        registerReceiver(receiver, intentFilter)
    }

    private val receiver = object : BroadcastReceiver() {
        override fun onReceive(context: Context, intent: Intent) {
            val mAction = intent.action
            if (mAction == WXPayEntryActivity.WECHAT_PAY_RESULTS) {
                getQueryResult()
            }
        }
    }

    /**
     * 银联 && 易宝 回调
     * */
    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        getQueryResult()
    }

    private fun getQueryResult() {
        Handler().postDelayed({
            val presenter = PayMoneyPresenterImpl(this, PayQueryResult::class.java)
            presenter.addListener(queryModel)
            val request = PayQueryRequest()
            request.uid = SharedPreferencesUtil.getInstance().uid
            request.pay_method = mPayMethod
            request.out_trade_no = mPayTn
            presenter.payQuery(request)
        }, 2000)
    }

    private val queryModel = object : GeneralViewInterface {
        override fun getCodeBeanFail(errorCode: Int) {
            setPayResult(FAIL, "支付结果确认中")
        }

        override fun getCodeObjectSuccess(bean: Any?) {
            val result = bean as PayQueryResult
            when (result.code) {
                CarEntity.SUCC -> {
                    if (result.status == "FAIL") setPayResult(FAIL, "支付失败") else setPayResult(SUCC, "支付成功")
                }
                else -> setPayResult(FAIL, "支付结果确认中")
            }
        }
    }

    private fun setPayResult(code: Int, msg: String) {
        val intent = Intent()
        intent.putExtra(RESULT_CODE, code)
        intent.putExtra(RESULT_MSG, msg)
        setResult(Activity.RESULT_OK, intent)
        finish()
    }

    override fun onDestroy() {
        super.onDestroy()
        try {
            dismissLoadingDialog()
        } catch (e: Exception) {
        }
    }

    companion object {
        const val RESULT_CODE = "RESULT_CODE"
        const val RESULT_MSG = "RESULT_MSG"
        const val SUCC = 1
        const val FAIL = 2
    }
}