package com.jiaoyinbrother.monkeyking.activity

import android.content.Intent
import android.net.Uri
import android.os.AsyncTask
import android.view.View
import android.view.View.OnClickListener
import android.widget.Button
import android.widget.TextView
import cn.jpush.android.api.JPushInterface
import com.google.gson.Gson
import com.jiaoyinbrother.monkeyking.CarEntity
import com.jiaoyinbrother.monkeyking.R
import com.jiaoyinbrother.monkeyking.bean.CheckVersionResult
import com.jiaoyinbrother.monkeyking.bean.ProjectBaseResult
import com.jiaoyinbrother.monkeyking.jpush.MKPushUtil
import com.jiaoyinbrother.monkeyking.network.AppSetting
import com.jiaoyinbrother.monkeyking.network.CarLib
import com.jiaoyinbrother.monkeyking.util.*
import com.jiaoyinbrother.monkeyking.util.NetUtil
import com.jiaoyinbrother.monkeyking.util.Util
import com.jybrother.sineo.library.base.BaseActivity
import com.jybrother.sineo.library.bean.UserDetailRequestBean
import com.jybrother.sineo.library.bean.WebViewConfigEntity
import com.jybrother.sineo.library.presenter.CheckVersionPresenterImpl
import com.jybrother.sineo.library.util.*
import com.jybrother.sineo.library.view.GeneralViewInterface
import kotlinx.android.synthetic.main.act_more.*

class SettingsActivity : BaseActivity(), OnClickListener {

    private var mClearCache: TextView? = null

    private val maseModel = object : GeneralViewInterface {
        override fun getCodeBeanFail(errorCode: Int) {
            LogUtil.e("checkVersionPresenter getCodeBeanFail")
            showErrorToast(errorCode)
        }

        override fun getCodeObjectSuccess(bean: Any) {
            LogUtil.e("getCodeObjectSuccess")
            val result = bean as CheckVersionResult
            if (result.code == CarEntity.SUCC) {
                CheckVersionUtil.checkVersion(this@SettingsActivity, result, true)
            } else {
                showToast(result.msg)
            }

        }
    }

    override fun getLayoutId(): Int {
        return R.layout.act_more
    }

    override fun initView() {
        mClearCache = findViewById(R.id.cache_status) as TextView

    }

    override fun initToolBar() {

    }

    override fun initListeners() {
        btn_push.isChecked = SharedPreferencesUtil.getInstance().isJPushOpen
        btn_push.setOnCheckedChangeListener { _, isChecked ->
            StatisticsUtil.onEvent(this,ConstantsUtil.MINE_SETTINGS_CLOSE_PUSH,"")
            if (isChecked) {
                JPushInterface.resumePush(this@SettingsActivity)
                SharedPreferencesUtil.getInstance().saveJPushStatus("OPEN")
                OldToastUtil.show(this@SettingsActivity, "已开启推送")
            } else {
                JPushInterface.stopPush(this@SettingsActivity)
                SharedPreferencesUtil.getInstance().saveJPushStatus("CLOSE")
                OldToastUtil.show(this@SettingsActivity, "已关闭推送")
            }
        }
        personal_info_relative.setOnClickListener(this)
        contact_relative.setOnClickListener(this)

        contact_us_relative.setOnClickListener(this)
        rate_relative.setOnClickListener(this)

        check_version_relative.setOnClickListener(this)
        clear_cache_relative.setOnClickListener(this)

        logout_relative.setOnClickListener(this)

    }

    override fun initData() {
        (findViewById(R.id.ivTitleName) as Button).text = "设置"
        findViewById(R.id.ivTitleBtnRight_ll).visibility = View.INVISIBLE


        // 获取许需要清除的缓存大小
        val cacheSize = FileUtils.getTotalCacheSize(this)
        cache_status.text = cacheSize

        version_name.text = "版本号V" + Util.getVersion(this@SettingsActivity)
    }

    /**
     * 点击事件
     */
    override fun onClick(v: View?) {
        when (v?.id) {
            R.id.personal_info_relative -> {
                val intent = Intent()
                intent.setClass(this@SettingsActivity, PersonalInfoActivity::class.java)
                startActivity(intent)
            }
            R.id.contact_relative -> {
                /* 紧急联系人 */
                StatisticsUtil.onEvent(this,ConstantsUtil.MINE_SETTINGS_CONTACTS, "")
                if (SharedPreferencesUtil.getInstance().isLogin) {
                    val intent = Intent()
                    intent.setClass(this@SettingsActivity, ContactsActivity::class.java)
                    startActivity(intent)
                } else {
                    val intent = Intent()
                    intent.setClass(this@SettingsActivity, LoginActivity::class.java)
                    startActivity(intent)
                }
            }


            R.id.contact_us_relative -> {
                StatisticsUtil.onEvent(this,ConstantsUtil.MINE_SETTINGS_CONTACT_US, "")
                handleContactUs()
            }
            R.id.rate_relative -> {
                StatisticsUtil.onEvent(this, ConstantsUtil.MINE_SETTINGS_APP_SCORE,"")
                handleMarket()
            }

            R.id.check_version_relative -> {
                handleCheckVersion()
            }
            R.id.clear_cache_relative -> {
                StatisticsUtil.onEvent(this, ConstantsUtil.MINE_SETTINGS_CLEAN_CACHE,"")
                /* 清除缓存 */
                handleClearCache()
            }
            R.id.logout_relative -> {
                SharedPreferencesUtil.getInstance().setString("refresh", "true")
                handleLogout()
            }
        }
    }

    private fun handleMarket() {
        val uri = Uri.parse("market://details?id=" + packageName)
        val intent = Intent(Intent.ACTION_VIEW, uri)
        intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
        try {
            startActivity(intent)
        } catch (e: Exception) {
            OldToastUtil.show(this@SettingsActivity, "没找到相关市场")
        }

    }

    private fun handleContactUs() {
        val webViewConfigEntity = WebViewConfigEntity()
        webViewConfigEntity.title = "联系我们"
        webViewConfigEntity.isNeedProgressBar = true
        webViewConfigEntity.url = AppSetting.getText_url() + "/#/profile/more/contact_us"
        webViewConfigEntity.isNeedShareButton = false

        val intent = Intent(this@SettingsActivity, GeneralWebViewActivity::class.java)
        intent.putExtra(CarEntity.FEATURE_ITEM, webViewConfigEntity)
        startActivity(intent)
    }

    /**
     * 清除缓存
     */
    private fun handleClearCache() {
        DeleteCacheAsyncTask().execute()
    }

    private inner class DeleteCacheAsyncTask : AsyncTask<Void, Void, Int>() {
        override fun doInBackground(vararg Integer: Void): Int? {
            FileUtils.clearWebViewCache(this@SettingsActivity)
            Util.clearImageLoader()
            GeneralWebViewActivity.clearCookies(this@SettingsActivity)
            return 1
        }

        override fun onPostExecute(temp: Int?) {
            super.onPostExecute(temp)
            val cacheSize = FileUtils.getTotalCacheSize(this@SettingsActivity)
            mClearCache!!.text = cacheSize
            OldToastUtil.show(this@SettingsActivity, "清除成功")
        }
    }

    /**
     * 版本检查
     */
    private fun handleCheckVersion() {
        val checkVersionPresenter = CheckVersionPresenterImpl(this, CheckVersionResult::class.java)
        checkVersionPresenter.addListener(maseModel)
        checkVersionPresenter.checkVersion(CheckVersionUtil.getCheckVersionEntity(this))

    }

    // 退出登录
    private fun handleLogout() {
        if (SharedPreferencesUtil.getInstance().isLogin) {
            LogoutAsyncTask().execute()

            //先logout,再更新Jpush
            AuthUtil.logout(this@SettingsActivity)
//            SharedPreferencesUtil.getInstance().logout()

            MKPushUtil.updateJPush(this@SettingsActivity)
            GeneralWebViewActivity.clearCookies(this@SettingsActivity)

//            val sharePreferenceTools = SharePreferenceTools(this)
//            SharePreferenceTools sharePreferenceTools = new SharePreferenceTools(this);
//            sharePreferenceTools.putBoolean(SharePreferenceTools.NEED_REFRESH_USER_DETAIL,true)

            val intent = Intent()
//            intent.putExtra("Logout", "Logout")
            setResult(LOGOUT_OK, intent)
            finish()
        }
    }


    /**
     * 退出登录
     */
    internal inner class LogoutAsyncTask : AsyncTask<Void, Void, ProjectBaseResult>() {
        override fun doInBackground(vararg params: Void): ProjectBaseResult {
            var mCarLib = CarLib.getInstance(applicationContext)
            val userDetailBean = UserDetailRequestBean()
            userDetailBean.uid = SharedPreferencesUtil.getInstance().getString(SharedPreferencesUtil.UID_KEY, "")

            var br = ProjectBaseResult()
            try {
                br = mCarLib.postRequest(Gson().toJson(userDetailBean), "user/logout", ProjectBaseResult::class.java) as ProjectBaseResult
            } catch (e: Exception) {
                NetUtil.handlePostException(br, e)
            }

            return br
        }

        override fun onPostExecute(result: ProjectBaseResult) {
            super.onPostExecute(result)


        }
    }


    companion object {
        val LOGOUT_OK = 100;
    }

}
