package com.jiaoyinbrother.monkeyking.activity

import android.app.Activity
import android.content.Intent
import android.os.Bundle
import android.os.Handler
import android.support.v4.content.ContextCompat
import android.text.TextUtils
import android.view.View
import com.jiaoyinbrother.monkeyking.R
import com.jybrother.sineo.library.base.BaseActivity
import kotlinx.android.synthetic.main.activity_taking_pictures.*
import com.jiaoyinbrother.monkeyking.CarEntity
import com.jiaoyinbrother.monkeyking.Constants
import com.jiaoyinbrother.monkeyking.fragment.DepositFragment
import com.jiaoyinbrother.monkeyking.fragment.DepositFragment.*
import com.jiaoyinbrother.monkeyking.fragment.TakingPicFragment
import com.jiaoyinbrother.monkeyking.fragment.TakingPicFragment.*
import com.jiaoyinbrother.monkeyking.util.SharedPreferencesUtil
import com.jiaoyinbrother.monkeyking.view.IOSdialog.IOSAlertDialog
import com.jybrother.sineo.library.adapter.MyFragmentAdapter
import com.jybrother.sineo.library.bean.*
import com.jybrother.sineo.library.view.GeneralViewInterface
import com.jybrother.sineo.library.presenter.CarImgPresenterImpl
import com.jybrother.sineo.library.presenter.OrderPresenterImpl
import com.jybrother.sineo.library.util.ConstantsUtil
import com.jybrother.sineo.library.util.TextUtil
import com.jybrother.sineo.library.widget.GeneralBottomButton.*
import java.util.*

/**
 * by Soda on 2018/1/22.
 */
class TakingPicturesActivity : BaseActivity() {

    private var depFragment: DepositFragment? = null
    private var picFragment: TakingPicFragment? = null
    private var orderResult: OrderDetailResult? = null
    private var deposit: DepositBean? = null

    private var fromFlag = FLAG_TAKE_PIC
    // post params
    private var orderID: String? = null
    private var postFlag: String? = null
    private var picUrlList: List<String>? = null
    private var fromSucc: String? = null

    override fun getLayoutId(): Int {
        getExtra()
        return R.layout.activity_taking_pictures
    }

    private fun getExtra() {
        fromFlag = intent?.getStringExtra(Constants.FLAG_TAKING_PIC).toString()
        orderResult = intent?.getSerializableExtra(Constants.KEY_ORDER_DETAIL) as OrderDetailResult
        orderID = orderResult?.orderid
        fromSucc = intent?.getStringExtra("fromSucc")
    }

    override fun initToolBar() {
        mainTitle.text = when (fromFlag) {
            FLAG_TAKE_PIC -> {
                if (orderResult?.take_car_image == null) "拍摄照片" else "取车照片"
            }
            FLAG_RETURN_PIC -> {
                "确认还车"
            }
            FLAG_RETURN_FINISH -> {
                "还车照片"
            }
            else -> "拍摄照片"
        }
    }

    override fun initView() {
        goSubmit.disable()// 初始按钮置灰

        val mFragmentAdapter = MyFragmentAdapter(supportFragmentManager)
        // 押金
        depFragment = DepositFragment()
        val depBundle = Bundle()
        depBundle.putString("fromFlag", fromFlag)
        depBundle.putSerializable(ConstantsUtil.ORDER_RESULT, orderResult)
        depFragment?.arguments = depBundle
        mFragmentAdapter.addFragment(depFragment)
        // 拍照
        picFragment = TakingPicFragment()
        val picBundle = Bundle()
        picBundle.putString("fromFlag", fromFlag)
        picBundle.putSerializable(ConstantsUtil.ORDER_RESULT, orderResult)
        picFragment?.arguments = picBundle
        mFragmentAdapter.addFragment(picFragment)

        viewPager.offscreenPageLimit = 2
        viewPager.adapter = mFragmentAdapter

        showLayout(fromFlag, orderResult)
    }

    override fun initListeners() {
        goSubmit.setListener(object : SubmitListener {
            override fun submit() {
                when (fromFlag) {
                    FLAG_TAKE_PIC -> {
                        upLoadPic()
                    }
                    FLAG_RETURN_PIC -> {
                        if (getIllegalDeposit(orderResult)) {// 押金已交付
                            upLoadPic()
                        } else {
                            val intent = Intent(this@TakingPicturesActivity, CashierDeskActivity::class.java)
                            intent.putExtra(ConstantsUtil.BUNDLE_EXTRA_KEY, ConstantsUtil.PAY_PROJECT_ILLEGAL)
                            intent.putExtra(ConstantsUtil.ORDER_RESULT, orderResult)
                            intent.putExtra("where are you from", "TakingPic")
                            startActivity(intent)
                        }
                    }
                }
            }
        })
        picFragment?.setListener(object : TakingPicListener {
            override fun activeSubmit(array: Array<String>) {
                picUrlList = Arrays.asList<String>(*array)
                val b: Boolean = (0 until array.size).none { TextUtils.isEmpty(array[it]) }
                if (b) {
                    runOnUiThread { goSubmit.normal() }
                } else {
                    runOnUiThread { goSubmit.disable() }

                }
            }
        })
        depFragment?.setListener(object : DepositListener {
            override fun onPay() {
                runOnUiThread { goSubmit.normal() }
            }
        })
    }

    override fun initData() {
        // 获取订单详情
        onPolling()
    }

    private fun orderDetail() {
        val presenter = OrderPresenterImpl(this, OrderDetailResult::class.java)
        presenter.addListener(mOrderDetailModel)
        val request = OrderDetailRequest()
        request.orderid = orderID
        request.uid = SharedPreferencesUtil.getInstance().uid
        presenter.getOrderDetail(request)
    }

    private val mOrderDetailModel = object : GeneralViewInterface {
        override fun getCodeBeanFail(errorCode: Int) {
        }

        override fun getCodeObjectSuccess(bean: Any) {
            val result = bean as OrderDetailResult
            when (result.code) {
                CarEntity.SUCC -> {
                    orderResult = result
                    picFragment?.setData(result, fromFlag)
                    depFragment?.setData(result)
                    showLayout(fromFlag, result)
                    when (fromFlag) {
                        FLAG_TAKE_PIC -> {
                            isPollRunning = false
                        }
                        FLAG_RETURN_PIC -> {
                            isPollRunning = false
                        }
                        FLAG_RETURN_FINISH -> {
                            if (result.return_car_image != null) {
                                when (result.return_car_image?.status_code) {
                                    0 -> {// failed
                                        isPollRunning = false
                                        onAuditResult(result)
                                    }
                                    1 -> {// success
                                        isPollRunning = false
                                        if (TextUtils.isEmpty(fromSucc)) {
                                            onAuditResult(result)
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
                else -> showToast(result.msg)
            }
        }
    }

    private fun onPolling() {
        if (isPollRunning)
            return
        isPollRunning = true
        handler.post(runnable)
    }

    private fun onAuditResult(result: OrderDetailResult) {
        val intent = Intent(this@TakingPicturesActivity, AuditResultActivity::class.java)
        intent.putExtra("AuditCode", result.return_car_image?.status_code ?: 0)
        intent.putExtra("OrderResult", result)
        startActivity(intent)
        finish()
    }

    var isPollRunning = false
    internal val handler = Handler()
    internal var runnable: Runnable = object : Runnable {
        override fun run() {
            if (!isPollRunning)
                return
            handler.postDelayed(this, POLL_TIME.toLong())// 30秒
            orderDetail()
        }
    }

    private fun showLayout(fromFlag: String?, result: OrderDetailResult?) {
        // 页面显示
        when (fromFlag) {
            FLAG_TAKE_PIC -> {
                postFlag = POST_TAKE
                viewPager.currentItem = PAGE_TAKING_PIC
                goSubmit.visibility = if (result?.take_car_image != null) View.GONE else View.VISIBLE
                goSubmit.setText("提交")
            }
            FLAG_RETURN_PIC -> {
                postFlag = POST_RETURN
                viewPager.currentItem = if (getIllegalDeposit(result)) PAGE_TAKING_PIC else PAGE_DEPOSIT
                val bottomBtn = if (getIllegalDeposit(result)) "提交" else "立即支付"
                goSubmit.setText(bottomBtn)
            }
            FLAG_RETURN_FINISH -> {
                viewPager.currentItem = PAGE_TAKING_PIC
                runOnUiThread { goSubmit.visibility = View.GONE }
                when (result?.return_car_image?.status_code) {
                    0 -> {// 失败
                        auditStatus.visibility = View.GONE
                    }
                    1 -> {// 通过
                        auditStatus.visibility = View.GONE
                    }
                    2 -> {// 中
                        mainTitle.text = "确认还车"
                        auditStatus.visibility = View.VISIBLE
                        val sp = "预计" + result.return_car_image?.approve_time + "完成审核"
                        auditStatus.text = TextUtil.setForegroundColor(sp, ContextCompat.getColor(
                                this@TakingPicturesActivity, R.color.color_6), 2, sp.length - 4)
                    }
                }
            }
            else -> {
                // 取车完成还是还车完成都只显示图片不能操作
                viewPager.currentItem = PAGE_TAKING_PIC
                goSubmit.visibility = View.GONE
            }
        }
    }

    private fun getIllegalDeposit(result: OrderDetailResult?): Boolean {
        result?.deposit?.filter { TextUtils.equals(it.type, ConstantsUtil.PAY_PROJECT_ILLEGAL) }
                ?.forEach { deposit = it }
        return deposit?.status_code == DepositBean.DEPOSIT_PAYED
    }

    override fun onBackPressed() {
        isPollRunning = false
        if (picUrlList == null) {
            super.onBackPressed()
        } else {
            if (orderResult?.return_car_image == null) {
                IOSAlertDialog(this@TakingPicturesActivity).builder().setCancelable(false).setTitle("温馨提示")
                        .setMsg("尚未完成拍照操作，是否退出？", true).setNegativeButton("取消") { }
                        .setPositiveButton("确认") { super.onBackPressed() }.show()
            } else {
                val intent = Intent(this@TakingPicturesActivity, OrderDetailActivity::class.java)
                intent.putExtra(ConstantsUtil.ORDER_ID, orderResult?.orderid)
                startActivity(intent)
                super.onBackPressed()
            }
        }
    }

    private fun upLoadPic() {
        val presenter = CarImgPresenterImpl(this@TakingPicturesActivity, TakingPicResult::class.java)
        presenter.addListener(upLoadPicModel)
        val request = TakingPicRequest()
        request.order_id = orderID
        request.type = postFlag
        request.pics = picUrlList
        presenter.applyCarImg(request)
    }

    private val upLoadPicModel = object : GeneralViewInterface {
        override fun getCodeBeanFail(errorCode: Int) {
            submitFail()
        }

        override fun getCodeObjectSuccess(bean: Any?) {
            val result = bean as TakingPicResult
            when (result.code) {
                CarEntity.SUCC -> {
                    when (fromFlag) {
                        FLAG_TAKE_PIC -> {
                            showToast("提交成功！")
                            val intent = Intent()
                            intent.putExtra(Constants.KEY_TAKING_PIC, "SUCC")
                            setResult(Activity.RESULT_OK, intent)
                            finish()
                        }
                        else -> {
                            val sp = "预计" + result.approve_time + "完成审核"
                            auditStatus.text = TextUtil.setForegroundColor(sp, ContextCompat.getColor(this@TakingPicturesActivity, R.color.color_6), 2, sp.length - 4)
                            IOSAlertDialog(this@TakingPicturesActivity).builder().setCancelable(false).setTitle("提交成功！").setMsg(result.tips, true).setPositiveButton("我知道了") {
                                fromFlag = FLAG_RETURN_FINISH
                                onPolling()
                            }.show()
                        }
                    }
                }
                else -> submitFail()
            }
        }
    }

    private fun submitFail() {
        IOSAlertDialog(this@TakingPicturesActivity).builder().setCancelable(false).setTitle("提交失败")
                .setMsg("照片提交失败，是否重试？", true).setNegativeButton("取消") { }
                .setPositiveButton("重试") { upLoadPic() }.show()
    }

    override fun onNewIntent(intent: Intent?) {
        super.onNewIntent(intent)
        if (intent?.getStringExtra("onNewIntent") == "onNewIntent") {
            orderResult = intent.getSerializableExtra(ConstantsUtil.ORDER_RESULT) as OrderDetailResult?
            goSubmit.disable()
            orderDetail()
        }
    }

    override fun onDestroy() {
        super.onDestroy()
        isPollRunning = false
    }

    companion object {
        const val FLAG_TAKE_PIC = "FLAG_TAKE_PIC"
        const val FLAG_RETURN_PIC = "FLAG_RETURN_PIC"
        const val FLAG_RETURN_FINISH = "FLAG_RETURN_FINISH"
        // post params
        const val POST_TAKE = "TAKE_CAR"
        const val POST_RETURN = "RETURN_CAR"
        //
        const val PAGE_DEPOSIT = 0
        const val PAGE_TAKING_PIC = 1

        const val POLL_TIME = 30000// 30秒
    }
}

