package com.jiaoyinbrother.monkeyking.adapter

import android.annotation.SuppressLint
import android.content.Context
import android.graphics.Color
import android.support.constraint.ConstraintLayout
import android.support.v4.content.ContextCompat
import android.text.*
import android.text.style.RelativeSizeSpan
import android.view.View
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.TextView
import com.camnter.easyrecyclerview.adapter.EasyRecyclerViewAdapter
import com.camnter.easyrecyclerview.holder.EasyRecyclerViewHolder
import com.jiaoyinbrother.monkeyking.R
import com.jiaoyinbrother.monkeyking.bean.CouponBean
import com.jiaoyinbrother.monkeyking.fragment.CouponFragment
import com.jybrother.sineo.library.util.LogUtil
import com.jybrother.sineo.library.util.TextUtil
import com.jybrother.sineo.library.widget.RoundBackgroundColorSpan
import java.text.SimpleDateFormat

/**
 * by Soda on 2018/3/5.
 */

class CouponAdapter(val context: Context, private val couponType: Int) : EasyRecyclerViewAdapter() {

    // 单选 isSelected 是否选中 singleSelect选中ID
    private var isSelected = false
    private var isSelectId = -1
    private var footer = false

    override fun getItemLayouts(): IntArray {
        return intArrayOf(R.layout.item_coupon, R.layout.item_footer)
    }

    override fun getRecycleViewItemType(position: Int): Int {
        return if (footer && position == itemCount - 1) 1 else 0
    }

    fun addFooter(footer: Boolean) {
        this.footer = footer
    }

    @SuppressLint("SetTextI18n")
    override fun onBindRecycleViewHolder(viewHolder: EasyRecyclerViewHolder, position: Int) {
        val recycleViewItemType = getRecycleViewItemType(position)
        when (recycleViewItemType) {
            0 -> couponViewHolder(viewHolder, position)
            1 -> footerViewHolder(viewHolder, position)
        }
    }

    private fun footerViewHolder(viewHolder: EasyRecyclerViewHolder, position: Int) {}

    @SuppressLint("SetTextI18n")
    private fun couponViewHolder(viewHolder: EasyRecyclerViewHolder, position: Int) {
        val coupon = list[position] as CouponBean
        // 金额
        val couponAmount = viewHolder.findViewById<TextView>(R.id.couponAmount)
        couponAmount.text = if (TextUtils.isEmpty(coupon.ratio)) {
            TextUtil.setForegroundSize("¥${coupon.amount}", 0.5f, 0, 1)
        } else {
            coupon.ratio
        }
        // 金额描述
        val couponAmountDesc = viewHolder.findViewById<TextView>(R.id.couponAmountDesc)
        couponAmountDesc.text = coupon.amount_desc
        // 有效期
        val couponDate = viewHolder.findViewById<TextView>(R.id.couponDate)
        couponDate.text = "${formatTime(coupon.start_date.toString())}至${formatTime(coupon.expdate.toString())}有效"
        // 左侧布局
        val couponLeftLayout = viewHolder.findViewById<LinearLayout>(R.id.couponLeftLayout)
        // 右侧布局
        val couponRightLayout = viewHolder.findViewById<ConstraintLayout>(R.id.couponRightLayout)
        // 券描述
        val couponDesc = viewHolder.findViewById<TextView>(R.id.couponDesc)
        val descBgColor = when (couponType) {
            CouponFragment.TYPE_USED -> Color.parseColor("#D2D1D1")
            CouponFragment.TYPE_OVERDUE -> Color.parseColor("#D2D1D1")
            else -> {
                if (isBeginning(coupon.start_date.toString()) || coupon.avai == 1) {
                    ContextCompat.getColor(context, R.color.color_6_new)
                } else {
                    Color.parseColor("#D2D1D1")
                }
            }
        }
        couponDesc.text = setCouponDesc("${coupon.name}${coupon.description}", coupon.name?.length ?: 0, descBgColor)
        // 状态图标
        val couponStatus = viewHolder.findViewById<ImageView>(R.id.couponStatus)
        setCouponBackground(couponLeftLayout, couponRightLayout, couponStatus, coupon)
        // 勾选状态
        val couponSelected = viewHolder.findViewById<ImageView>(R.id.couponSelected)
        // 单选状态
        singleSelected(couponRightLayout, couponSelected, position)
    }

    /**
     * 单选状态显示
     * */
    private fun singleSelected(rightLayout: ConstraintLayout, ivSelected: ImageView, position: Int) {
        if (isSelectId == position && !isSelected) {
            rightLayout.setBackgroundResource(R.mipmap.bg_coupon_right_seleted)
            ivSelected.visibility = View.VISIBLE
        } else {
            rightLayout.setBackgroundResource(R.mipmap.bg_coupon_right)
            ivSelected.visibility = View.GONE
        }
    }

    private fun setCouponBackground(leftLayout: LinearLayout, rightLayout: ConstraintLayout, status: ImageView, coupon: CouponBean) {
        when (couponType) {
            CouponFragment.TYPE_USED -> {
                val bg = if (TextUtils.equals(coupon.category, "ZUCHE")) {
                    R.mipmap.bg_coupon_left_disable_zc
                } else {
                    R.mipmap.bg_coupon_left_disable_zjy
                }
                leftLayout.setBackgroundResource(bg)
                status.visibility = View.VISIBLE
                status.setBackgroundResource(R.mipmap.ic_coupon_used)
            }
            CouponFragment.TYPE_OVERDUE -> {
                val bg = if (TextUtils.equals(coupon.category, "ZUCHE")) {
                    R.mipmap.bg_coupon_left_disable_zc
                } else {
                    R.mipmap.bg_coupon_left_disable_zjy
                }
                leftLayout.setBackgroundResource(bg)
                status.visibility = View.VISIBLE
                status.setBackgroundResource(R.mipmap.ic_coupon_overdue)
            }
            else -> {
                status.visibility = View.GONE
                // 开始时间在当前之前或者相等 || avai == 1 优惠券标识可以使用
                val bg = if (isBeginning(coupon.start_date.toString()) || coupon.avai == 1) {
                    if (TextUtils.equals(coupon.category, "ZUCHE")) {
                        R.mipmap.bg_coupon_left_zc
                    } else {
                        R.mipmap.bg_coupon_left_zjy
                    }
                } else {
                    if (TextUtils.equals(coupon.category, "ZUCHE")) {
                        R.mipmap.bg_coupon_left_disable_zc
                    } else {
                        R.mipmap.bg_coupon_left_disable_zjy
                    }
                }
                leftLayout.setBackgroundResource(bg)
            }
        }
    }

    private fun isBeginning(stateDate: String): Boolean {
        // 判断是否可用
        val sdf = SimpleDateFormat("yyyy-MM-dd HH:mm:ss")
        var startDateTime: Long = 0
        try {
            startDateTime = sdf.parse(stateDate).time
        } catch (e: Exception) {
            LogUtil.e("setCouponBackground Exception")
        }
        return startDateTime <= System.currentTimeMillis()
    }

    /**
     * 优惠券描述内容 & 优惠券类型背景颜色
     * */
    private fun setCouponDesc(desc: String, endIndex: Int, bgColor: Int): SpannableString {
        val sp = SpannableString(desc)
        val sSpan = RelativeSizeSpan(0.8f)
        sp.setSpan(sSpan, 0, endIndex, Spannable.SPAN_INCLUSIVE_EXCLUSIVE)
        val rbcSpan = RoundBackgroundColorSpan(context, bgColor, ContextCompat.getColor(context, R.color.color_0))
        sp.setSpan(rbcSpan, 0, endIndex, Spannable.SPAN_EXCLUSIVE_EXCLUSIVE)
        return sp
    }

    /**
     * 时间格式化
     */
    @SuppressLint("SimpleDateFormat")
    private fun formatTime(date: String): String {
        val sdf = SimpleDateFormat("yyyy-MM-dd HH:mm:ss")
        var time: Long = 0
        return try {
            val d = sdf.parse(date)
            time = d.time
            SimpleDateFormat("yyyy-MM-dd").format(time)
        } catch (ex: Exception) {
            "yyyy-MM-dd"
        }
    }

    /**
     * 单选
     */
    fun setSelected(position: Int) {
        LogUtil.e("selectedId =====" + isSelectId)
        isSelected = if (isSelectId == position) !isSelected else false
        LogUtil.e("isSelected  ===== " + isSelected)
        isSelectId = position
        notifyDataSetChanged()
    }


}
