package com.jiaoyinbrother.monkeyking.adapter

import android.annotation.SuppressLint
import android.content.Context
import android.support.v4.content.ContextCompat
import android.text.TextUtils
import android.view.View
import android.widget.LinearLayout
import android.widget.TextView
import com.camnter.easyrecyclerview.adapter.EasyRecyclerViewAdapter
import com.camnter.easyrecyclerview.holder.EasyRecyclerViewHolder
import com.facebook.drawee.view.SimpleDraweeView
import com.jiaoyinbrother.monkeyking.R
import com.jiaoyinbrother.monkeyking.util.SharedPreferencesUtil
import com.jybrother.sineo.library.bean.OrdersBean
import com.jybrother.sineo.library.util.DateTimeUtil
import com.jybrother.sineo.library.util.LogUtil
import com.jybrother.sineo.library.widget.OrderButton
import java.text.SimpleDateFormat

/**
 * by Soda on 2018/3/27.
 */
class OrderListAdapter(val context: Context) : EasyRecyclerViewAdapter() {

    private var footer = false
    private var listener: OrderItemClickListener? = null


    override fun getItemLayouts(): IntArray {
        return intArrayOf(R.layout.item_order, R.layout.item_footer)
    }

    override fun onBindRecycleViewHolder(viewHolder: EasyRecyclerViewHolder?, position: Int) {

        val recycleViewItemType = getRecycleViewItemType(position)
        when (recycleViewItemType) {
            0 -> orderViewHolder(viewHolder, position)
            1 -> footerViewHolder(viewHolder, position)
        }
    }

    @SuppressLint("SetTextI18n")
    private fun orderViewHolder(viewHolder: EasyRecyclerViewHolder?, position: Int) {
        val order = list[position] as OrdersBean
        // 订单号
        val orderNo = viewHolder?.findViewById<TextView>(R.id.orderNo)
        orderNo?.text = "订单号：${order.orderid}"
        // 订单日期
        val orderDate = viewHolder?.findViewById<TextView>(R.id.orderDate)
        orderDate?.text = if (TextUtils.isEmpty(order.submit_time)) "yyyy-MM-dd HH:mm" else formatTime(order.submit_time)
        // 订单渠道 (车管端)
        val orderChannel = viewHolder?.findViewById<TextView>(R.id.orderChannel)
        orderChannel?.visibility = if (SharedPreferencesUtil.getInstance().role == 1) View.VISIBLE else View.INVISIBLE
        orderChannel?.text = if (TextUtils.isEmpty(order.channel)) "" else " (${order.channel})"
        // 订单标红 (车管端)
        val layoutOrderInfo = viewHolder?.findViewById<LinearLayout>(R.id.layoutOrderInfo)
        redTagInfo(layoutOrderInfo, orderNo, orderChannel, orderDate, order)
        // 车辆图片
        val carImage = viewHolder?.findViewById<SimpleDraweeView>(R.id.carImg)
        carImage?.setImageURI(order.pic)
        // 车辆类别
        val carType = viewHolder?.findViewById<TextView>(R.id.carType)
        carType?.text = order.type
        // 车辆牌号
        val carNumber = viewHolder?.findViewById<TextView>(R.id.carNumber)
        carNumber?.visibility = if (SharedPreferencesUtil.getInstance().role == 0 && order.status < OrdersBean.ORDER_STATUS_5) View.GONE else View.VISIBLE
        carNumber?.text = order.number
        // 订单状态
        val orderStatus = viewHolder?.findViewById<TextView>(R.id.orderStatus)
        orderStatus?.text = order.status_name
        // 智能钥匙开关 (已经隐藏掉了)
//        val carRemote = viewHolder?.findViewById<RadioButton>(R.id.carRemote)
        // 取车时间
        val orderTakeTime = viewHolder?.findViewById<TextView>(R.id.orderTakeTime)
        orderTakeTime?.text = order.start_time
        // 取车地点
        val orderTakeAddress = viewHolder?.findViewById<TextView>(R.id.orderTakeAddress)
        orderTakeAddress?.text = order.address
        // 还车时间
        val orderReturnTime = viewHolder?.findViewById<TextView>(R.id.orderReturnTime)
        orderReturnTime?.text = order.end_time
        // 还车地点
        val orderReturnAddress = viewHolder?.findViewById<TextView>(R.id.orderReturnAddress)
        orderReturnAddress?.text = if (TextUtils.isEmpty(order.return_address)) order.address else order.return_address

        val orderButton = viewHolder?.findViewById<OrderButton>(R.id.orderButton)
        val role = SharedPreferencesUtil.getInstance().role
        if (role == 0)
            displayButton(order, orderButton)
        else
            orderButton?.visibility = View.GONE
        orderButton?.setInfo(order, role)
        orderButton?.setListener(object : OrderButton.OrderBtnListener {
            override fun smallOne(mStatus: Int?) {
                listener?.onSmallOne(order, position)
                LogUtil.e("smallOne---$position")
            }

            override fun smallTwo(mStatus: Int?) {
                listener?.onSmallTwo(order, position)
                LogUtil.e("smallTwo---$position")
            }

            override fun bigThree(mStatus: Int?) {
                listener?.onBigThree(order, position)
                LogUtil.e("bigThree---$position")
            }
        })

        LogUtil.e("item __ setData finish -----------------------------------------------------")
    }

    /**
     * 订单title标红
     * 当处于待交车&& 已经超过交车时间的订单
     * */
    private fun redTagInfo(layoutOrderInfo: LinearLayout?, orderNo: TextView?, orderChannel: TextView?, orderDate: TextView?, order: OrdersBean) {
        val role = SharedPreferencesUtil.getInstance().role
        val nowTime = System.currentTimeMillis()
        val startTime = getDateTime(order.start_time)
        val endTime = getDateTime(order.end_time)
        if ((role == 1 && order.status == OrdersBean.ORDER_STATUS_4 && nowTime > startTime)
                || (role == 1 && order.status == OrdersBean.ORDER_STATUS_5 && nowTime > endTime)) {
            layoutOrderInfo?.setBackgroundResource(R.color.color_6)
            orderNo?.setTextColor(ContextCompat.getColor(context, R.color.color_0))
            orderChannel?.setTextColor(ContextCompat.getColor(context, R.color.color_0))
            orderDate?.setTextColor(ContextCompat.getColor(context, R.color.color_0))
        } else {
            layoutOrderInfo?.setBackgroundResource(R.color.color_0)
            orderNo?.setTextColor(ContextCompat.getColor(context, R.color.color_5))
            orderChannel?.setTextColor(ContextCompat.getColor(context, R.color.color_3))
            orderDate?.setTextColor(ContextCompat.getColor(context, R.color.color_3))
        }
    }

    /**
     * 时间格式化
     */
    @SuppressLint("SimpleDateFormat")
    private fun formatTime(date: String): String {
        val sdf = SimpleDateFormat("yyyy-MM-dd HH:mm:ss")
        return try {
            val d = sdf.parse(date)
            val time = d.time
            SimpleDateFormat("yyyy-MM-dd HH:mm").format(time)
        } catch (ex: Exception) {
            "yyyy-MM-dd HH:mm"
        }
    }


    private fun getDateTime(date: String): Long {
        val sdf = SimpleDateFormat("yyyy-MM-dd HH:mm")
        return try {
            val d = sdf.parse(date)
            d.time
        } catch (ex: Exception) {
            0
        }
    }

    private fun footerViewHolder(viewHolder: EasyRecyclerViewHolder?, position: Int) {}


    override fun getRecycleViewItemType(position: Int): Int {
        return if (footer && position == itemCount - 1) 1 else 0
    }

    fun addFooter(footer: Boolean) {
        this.footer = footer
    }

    fun onItemClick(l: OrderItemClickListener) {
        this.listener = l
    }

    interface OrderItemClickListener {
        fun onSmallOne(order: OrdersBean, position: Int)
        fun onSmallTwo(order: OrdersBean, position: Int)
        fun onBigThree(order: OrdersBean, position: Int)
    }

    private fun displayButton(order: OrdersBean, bottom: OrderButton?) {
        when (order.status) {
            OrdersBean.ORDER_STATUS_0 -> bottom?.visibility = View.GONE
            OrdersBean.ORDER_STATUS_1 -> bottom?.visibility = View.VISIBLE
            OrdersBean.ORDER_STATUS_2 -> bottom?.visibility = View.GONE
            OrdersBean.ORDER_STATUS_3 -> bottom?.visibility = View.VISIBLE
            OrdersBean.ORDER_STATUS_4 -> bottom?.visibility = View.GONE
            OrdersBean.ORDER_STATUS_5 -> bottom?.visibility = if (DateTimeUtil.compareBigToDuration(DateTimeUtil.getCurrentTime(System.currentTimeMillis()), order.end_time)) View.VISIBLE else View.GONE
            OrdersBean.ORDER_STATUS_6 -> bottom?.visibility = View.VISIBLE
            OrdersBean.ORDER_STATUS_10 -> bottom?.visibility = if (order.markid != 0) View.GONE else View.VISIBLE
            else -> bottom?.visibility = if (order.markid != 0) View.GONE else View.VISIBLE
        }
    }
}