package com.jiaoyinbrother.monkeyking.adapter

import android.app.Activity
import android.content.Context
import android.view.View
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.TextView
import com.camnter.easyrecyclerview.adapter.EasyRecyclerViewAdapter
import com.camnter.easyrecyclerview.holder.EasyRecyclerViewHolder
import com.jiaoyinbrother.monkeyking.R
import com.jiaoyinbrother.monkeyking.activity.SiteListActivity
import com.jiaoyinbrother.monkeyking.util.LocationUtil
import com.jybrother.sineo.library.bean.SiteBean2
import com.jybrother.sineo.library.util.DensityUtil
import com.jybrother.sineo.library.util.LogUtil

/**
 * The Author : Soda
 * Created on 2017/12/21.
 */

class SiteAdapter(private val context: Context) : EasyRecyclerViewAdapter() {

    private var listener: ItemClick? = null

    override fun getItemLayouts(): IntArray {
        return intArrayOf(R.layout.item_site_title, R.layout.item_site_detail)
    }

    override fun getRecycleViewItemType(position: Int): Int {
        val siteBean = list[position] as SiteBean2
        return if (siteBean.itemType == SiteBean2.TYPE_TITLE) NORMAL else TITLE
    }

    override fun onBindRecycleViewHolder(viewHolder: EasyRecyclerViewHolder?, position: Int) {
        LogUtil.e("site adapter--------------------------------------------------$position")
        val itemType = getRecycleViewItemType(position)

        when (itemType) {
            0 -> areaHolder(viewHolder, position)
            1 -> siteHolder(viewHolder, position)
        }
    }

    private fun areaHolder(holder: EasyRecyclerViewHolder?, position: Int) {
        val siteBean = list[position] as SiteBean2

        val mainView = holder?.findViewById<LinearLayout>(R.id.siteTitleLayout)
        val params = LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT, DensityUtil.dip2px(context, 33F))
        mainView?.layoutParams = params

        val mTitle = holder?.findViewById<TextView>(R.id.siteTitle)
        mTitle?.text = siteBean.area
    }

    private fun siteHolder(holder: EasyRecyclerViewHolder?, position: Int) {
        val siteBean = list[position] as SiteBean2

        // site 名称
        val siteName = holder?.findViewById<TextView>(R.id.siteName)
        siteName?.text = siteBean.name
        // site 标签
        val siteTag = holder?.findViewById<LinearLayout>(R.id.siteTag)
        val tagList = siteBean.icon_tags
        siteTag?.visibility = View.GONE
        siteTag?.removeAllViews()
        if (tagList != null) {
            for (tag in tagList) {
                siteTag?.addView(addViews(tag, context))
            }
            siteTag?.visibility = View.VISIBLE
        }
        // site 距离
        val siteRange = holder?.findViewById<TextView>(R.id.siteRange)
        siteRange?.text = LocationUtil.calcLocation(siteBean.distance)
        siteRange?.visibility = if (siteBean.area == SiteListActivity.NEARBY_AREA) View.VISIBLE else View.GONE
        // site 详情
        val siteDetail = holder?.findViewById<ImageView>(R.id.siteDetail)
        siteDetail?.visibility = if (siteBean.area == SiteListActivity.NEARBY_AREA) View.GONE else View.VISIBLE
        val endLayout = holder?.findViewById<LinearLayout>(R.id.endLayout)
        endLayout?.setOnClickListener({ listener?.itemClick(siteBean) })
        // site 地址
        val siteAddress = holder?.findViewById<TextView>(R.id.siteAddress)
        siteAddress?.text = siteBean.address

        val screenWidth = (context as Activity).windowManager.defaultDisplay.width - DensityUtil.dip2px(context, 120f)
        // 下面代码是用来计算控件宽度的为了满足产品的无聊需求
        val w = View.MeasureSpec.makeMeasureSpec(0, View.MeasureSpec.UNSPECIFIED)
        val h = View.MeasureSpec.makeMeasureSpec(0, View.MeasureSpec.UNSPECIFIED)
        // 尾部布局宽度
        val endWidth = DensityUtil.dip2px(context, 45f)
        // 标签布局宽度
        siteTag?.measure(w, h)
        val tagWidth = siteTag?.measuredWidth ?: 0
        // site 名称最大宽度，用来设置名称尾部省略
        siteName?.maxWidth = screenWidth - endWidth - tagWidth

    }


    companion object {
        val TITLE = 1
        val NORMAL = 0

        fun addViews(tag: String, context: Context): ImageView {
            val img = ImageView(context)
            var lp = LinearLayout.LayoutParams(DensityUtil.dip2px(context, 14f), DensityUtil.dip2px(context, 14f))
            var resource = 0
            when (tag) {
                "品牌" -> {
                    resource = R.mipmap.ic_tag_brand
                    lp = LinearLayout.LayoutParams(DensityUtil.dip2px(context, 21f), DensityUtil.dip2px(context, 12f))
                }
                "24h" -> {
                    resource = R.mipmap.ic_tag_24h
                }
                "充电桩" -> {
                    resource = R.mipmap.ic_tag_charge
                }
                "机场" -> {
                    resource = R.mipmap.ic_tag_airport
                }
                "火车站" -> {
                    resource = R.mipmap.ic_tag_train
                }
            }
            img.setBackgroundResource(resource)
            lp.setMargins(6, 0, 0, 0)
            img.layoutParams = lp
            return img
        }
    }

    fun setListener(listener: ItemClick) {
        this.listener = listener
    }

    interface ItemClick {
        fun itemClick(siteBean: SiteBean2)
    }

}