package com.jiaoyinbrother.monkeyking.fragment

import android.os.Bundle
import android.text.TextUtils
import android.view.View
import com.camnter.easyrecyclerview.holder.EasyRecyclerViewHolder
import com.jiaoyinbrother.monkeyking.CarEntity
import com.jiaoyinbrother.monkeyking.R
import com.jiaoyinbrother.monkeyking.adapter.CouponAdapter
import com.jiaoyinbrother.monkeyking.bean.AddCouponsResult
import com.jiaoyinbrother.monkeyking.bean.CouponBean
import com.jiaoyinbrother.monkeyking.bean.CouponsResult
import com.jiaoyinbrother.monkeyking.util.NetUtil
import com.jiaoyinbrother.monkeyking.util.SharedPreferencesUtil
import com.jiaoyinbrother.monkeyking.util.Util
import com.jybrother.sineo.library.base.BaseFragment
import com.jybrother.sineo.library.bean.AddCouponRequest
import com.jybrother.sineo.library.bean.CouponsRequest
import com.jybrother.sineo.library.presenter.CouponPresenterImpl
import com.jybrother.sineo.library.util.ConstantsUtil
import com.jybrother.sineo.library.util.ErrorToast
import com.jybrother.sineo.library.util.LogUtil
import com.jybrother.sineo.library.util.TextUtil
import com.jybrother.sineo.library.view.GeneralViewInterface
import kotlinx.android.synthetic.main.fragment_coupon.*

/**
 * by Soda on 2018/2/26.
 */
class CouponFragment : BaseFragment(), View.OnClickListener, EasyRecyclerViewHolder.OnItemClickListener {

    companion object {
        val TYPE_UNUSED = 0
        val TYPE_USED = 1
        val TYPE_OVERDUE = 2
        val TYPE_CHOOSE = 3
        private val PAGE_SIZE = 10
        private val NET_ERROR = 1
        private val NO_COUPON = 2
    }

    private var couponType = 0
    private var page = 1

    private var mAdapter: CouponAdapter? = null
    private var usableCoupons: ArrayList<CouponBean>? = null
    private var defSelectedCoupon: ArrayList<String>? = null
    private var selectedCoupon = ArrayList<CouponBean>()
    private var mData: ArrayList<CouponBean>? = null

    @Suppress("UNCHECKED_CAST")
    override fun getLayoutId(): Int {
        couponType = arguments.getInt("STATUS_TYPE", 0)
        LogUtil.e("extra couponType :$couponType")
        usableCoupons = arguments.getSerializable("usableCoupons") as ArrayList<CouponBean>?
        defSelectedCoupon = arguments.getSerializable("defSelectedCoupon") as ArrayList<String>?
        return R.layout.fragment_coupon
    }

    override fun initViews(view: View?, savedInstanceState: Bundle?) {}

    override fun initToolbar(savedInstanceState: Bundle?) {}

    override fun initListeners() {
        mAdapter?.setOnItemClickListener(this)
        btnAddCoupons.setOnClickListener(this)
        refreshLayout.setRefreshListener { if (couponType < TYPE_CHOOSE) refreshCoupons() else refreshLayout.refreshIsEnd() }
        refreshLayout.setMoreListener { if (couponType < TYPE_CHOOSE) getCoupons(couponType) else refreshLayout.loadMoreIsEnd() }
    }

    override fun initData() {
        mData = ArrayList()
        refreshLayout.setFooter()
        addCouponLayout.visibility = if (couponType == TYPE_UNUSED) View.VISIBLE else View.GONE // 添加优惠券布局
        mAdapter = CouponAdapter(activity, couponType)
        recyclerView.adapter = mAdapter
        when (couponType) {
            TYPE_CHOOSE -> getBookCoupons()
            else -> getCoupons(couponType)
        }

    }

    override fun onClick(v: View?) {
        when (v?.id) {
            R.id.btnAddCoupons -> addCoupon(edAddCoupons.text.toString())
        }
    }

    override fun onItemClick(view: View?, position: Int) {
        if (couponType == TYPE_CHOOSE) {
            val coupon = mAdapter?.getItem<CouponBean>(position)
            if (coupon != null && coupon.avai == 1) {
                if (!Util.isListEmpty(selectedCoupon) && TextUtils.equals(selectedCoupon[0].id, coupon.id.toString())) {
                    selectedCoupon.remove(coupon)
                    chooseListener?.onChooseCancel()
                } else {
                    selectedCoupon.clear()
                    selectedCoupon.add(coupon)
                    chooseListener?.onChooseResult(coupon)
                }
                mAdapter?.setSelected(position)
            }
        }
    }

    private fun refreshCoupons() {
        page = 1
        getCoupons(couponType)
    }

    private fun getCoupons(type: Int) {
        val presenter = CouponPresenterImpl(activity, CouponsResult::class.java)
        presenter.addListener(couponsListModel)
        val request = CouponsRequest()
        request.uid = SharedPreferencesUtil.getInstance().uid
        request.page = page
        request.page_size = PAGE_SIZE
        request.status = type
        presenter.getCoupons(request)
    }

    private var isFooterAdded = false

    private val couponsListModel = object : GeneralViewInterface {
        override fun getCodeBeanFail(errorCode: Int) {
            if (activity == null || activity.isDestroyed || activity.isFinishing) {
                return;
            }
            refreshLayout.refreshIsEnd()
            refreshLayout.loadMoreIsEnd()
            showErrorToast(errorCode)
            emptyLayout(true, NET_ERROR)
        }

        override fun getCodeObjectSuccess(bean: Any?) {
            if (activity == null || activity.isDestroyed || activity.isFinishing) {
                LogUtil.e("activity == null || activity.isDestroyed || activity.isFinishing")
                return;
            }
            refreshLayout.refreshIsEnd()
            refreshLayout.loadMoreIsEnd()
            val result = bean as CouponsResult
            when (result.code) {
                CarEntity.SUCC -> {
                    countListener?.onCountResult(result.unused, result.used, result.expired)

//                    第一页，清除数据，footer
                    if (page == 1) {
                        mData?.clear()
                        deleteFooter()
                    }
                    mData?.addAll(result.coupons ?: ArrayList())

                    if (result.coupons.size < PAGE_SIZE) {
//                        最后的数据
                        if (result.coupons.size > 0) {
                            addFooter()
                        } else {
//                            个数为0， 如果是第一页，空页面；不是第一页，添加footer
                            if (page > 1) {
                                addFooter()
                            }
                        }
                    } else {
                        // 还有数据
                    }
                    mAdapter?.list = mData
                    mAdapter?.notifyDataSetChanged()
                    page++
                }
                else -> showToast(result.msg)
            }
            if (mAdapter?.itemCount == 0) emptyLayout(true, NO_COUPON) else emptyLayout(false, NO_COUPON)
        }
    }

    private fun deleteFooter() {
        mAdapter?.addFooter(false)
        isFooterAdded = false
    }

    private fun addFooter() {
        if (isFooterAdded)
            return;

        mAdapter?.addFooter(true)
        val footer = CouponBean()// bug 重复添加
        mData?.add(footer)
        isFooterAdded = true
    }

    private fun addCoupon(couponNumber: String) {
        if (!NetUtil.isNetworkAvailable(activity)) {
            showErrorToast(ErrorToast.LOCATION_TIMEOUT)
            return
        }
        if (TextUtils.isEmpty(couponNumber)) {
            showToast("未输入兑换码")
            return
        }
        val patter1 = "^[a-zA-Z0-9]{8}$"
        val patter2 = "^[a-zA-Z0-9]{10}$"
        if (!TextUtil.matchers_input(patter1, couponNumber) && !TextUtil.matchers_input(patter2, couponNumber)) {
            showToast("兑换码错误")
            return
        }
        showLoadingDialog()
        val presenter = CouponPresenterImpl(activity, AddCouponsResult::class.java)
        presenter.addListener(addCouponModel)
        val request = AddCouponRequest()
        request.uid = SharedPreferencesUtil.getInstance().uid
        request.couponcode = couponNumber
        presenter.addCoupon(request)
    }

    private val addCouponModel = object : GeneralViewInterface {
        override fun getCodeBeanFail(errorCode: Int) {
            dismissLoadingDialog()
            showErrorToast(errorCode)
        }

        override fun getCodeObjectSuccess(bean: Any?) {
            dismissLoadingDialog()
            val result = bean as AddCouponsResult
            when (result.code) {
                CarEntity.SUCC -> {
                    edAddCoupons.text = null
                    refreshCoupons()
                    showToast("兑换成功，优惠券已放入您的账户")
                }
                else -> showToast(result.msg)
            }
        }
    }

    private fun emptyLayout(b: Boolean, i: Int) {
        refreshLayout.visibility = if (b) View.GONE else View.VISIBLE
        couponEmpty.visibility = if (b) View.VISIBLE else View.GONE
        when (i) {
            NET_ERROR -> {
                couponEmpty.setEmptyData(R.mipmap.ic_no_network, "", ConstantsUtil.NO_NETWORK, "")
                couponEmpty.setNegativeButton("重新加载", View.OnClickListener {
                    refreshCoupons()
                })
            }
            NO_COUPON -> {
                val emptyRes = when (couponType) {
                    TYPE_UNUSED -> ConstantsUtil.NO_COUPON_UNUSED
                    TYPE_USED -> ConstantsUtil.NO_COUPON_USED
                    TYPE_OVERDUE -> ConstantsUtil.NO_COUPON_OVERED
                    else -> ConstantsUtil.NO_COUPON_BOOK
                }
                couponEmpty.setEmptyData(R.mipmap.ic_no_coupons, "", emptyRes, "")
            }
        }
    }

    private fun getBookCoupons() {
        try {
            mAdapter?.list = usableCoupons
            mAdapter?.notifyDataSetChanged()
            for (i in usableCoupons?.indices!!) {
                (defSelectedCoupon)?.filter { TextUtils.equals(usableCoupons?.get(i)?.id, it) }?.forEach {
                    mAdapter?.setSelected(i)
                    usableCoupons?.get(i)?.let { it1 -> selectedCoupon.add(it1) }
                }
            }
            if (mAdapter?.itemCount == 0) emptyLayout(true, NO_COUPON)
        } catch (e: Exception) {
        }
    }

    /**
     * coupons count call back
     * */
    interface CouponCountListener {
        fun onCountResult(unused: Int, used: Int, overdue: Int)
    }

    private var countListener: CouponCountListener? = null

    fun setCouponCount(l: CouponCountListener) {
        this.countListener = l
    }

    /**
     * coupon choose call back
     * */
    interface ChooseListener {
        fun onChooseResult(result: CouponBean)

        fun onChooseCancel()
    }

    private var chooseListener: ChooseListener? = null

    fun setChooseListener(l: ChooseListener) {
        this.chooseListener = l
    }
}