package com.jiaoyinbrother.monkeyking.fragment

import android.os.Bundle
import android.view.View
import com.jiaoyinbrother.monkeyking.CarEntity
import com.jiaoyinbrother.monkeyking.R
import com.jiaoyinbrother.monkeyking.adapter.OrderListAdapter
import com.jybrother.sineo.library.bean.OrdersBean
import com.jiaoyinbrother.monkeyking.util.SharedPreferencesUtil
import com.jybrother.sineo.library.base.BaseFragment
import com.jybrother.sineo.library.bean.OrdersRequest
import com.jybrother.sineo.library.bean.OrdersResult
import com.jybrother.sineo.library.presenter.OrderPresenterImpl
import com.jybrother.sineo.library.util.ConstantsUtil
import com.jybrother.sineo.library.view.GeneralViewInterface
import kotlinx.android.synthetic.main.fragment_order_list.*
import java.util.*
import kotlin.collections.ArrayList
import com.jybrother.sineo.library.util.LogUtil
import com.jybrother.sineo.library.widget.OrderListSortView


/**
 * by Soda on 2018/3/27.
 */
class OrderListFragment : BaseFragment() {

    private var orderType = 0
    private var page = 1
    private var sortFactor = OrderListSortView.DEFAULT
    private var sidx = ""// 排序字段(startTime, endTime)
    private var sord = ""// 排序方式(ASC,DESC)
    private var isFooterAdded = false

    private var mOrders: ArrayList<OrdersBean>? = null
    private var statusList = ArrayList<String>()
    private var mAdapter: OrderListAdapter? = null
    private var listener: OrderItemViewListener? = null

    override fun getLayoutId(): Int {
        orderType = arguments.getInt("ORDER_STATUS", 0)
        return R.layout.fragment_order_list
    }

    override fun initViews(view: View?, savedInstanceState: Bundle?) {}

    override fun initToolbar(savedInstanceState: Bundle?) {}

    override fun initListeners() {
        refreshLayout.setRefreshListener {
            page = 1
            getOrders()
        }
        refreshLayout.setMoreListener {
            getOrders()
        }
        mAdapter?.setOnItemClickListener { view, position ->
            val order = mAdapter?.getItem<OrdersBean>(position)
            listener?.onItemClick(order)
        }
        mAdapter?.onItemClick(object : OrderListAdapter.OrderItemClickListener {

            override fun onSmallOne(order: OrdersBean, position: Int) {
                listener?.onSmallOne(order, position)
            }

            override fun onSmallTwo(order: OrdersBean, position: Int) {
                listener?.onSmallTwo(order, position)
            }

            override fun onBigThree(order: OrdersBean, position: Int) {
                listener?.onBigThree(order, position)
            }
        })
    }

    override fun initData() {
        mOrders = ArrayList()
        refreshLayout.setFooter()
        LogUtil.e("initData orderType = $orderType")
        mAdapter = OrderListAdapter(activity)
        recyclerView.adapter = mAdapter
        statusList = getOrderStatusList(orderType)
        LogUtil.e("initData statusList = $statusList")
    }

    private fun getOrderStatusList(type: Int): ArrayList<String> {
        return when (type) {
        // 车管
            STATUS_MANAGER_HANDOVER -> ArrayList(Arrays.asList("ORDER_ACCEPTED", "ORDER_CAR_PAYED", "ORDER_ILLEGAL_PAYED"))
            STATUS_MANAGER_GOING -> ArrayList(Arrays.asList("ORDER_EXECUTING"))
            STATUS_MANAGER_SETTLED -> ArrayList(Arrays.asList("ORDER_BILL_APPLIED"))
            STATUS_MANAGER_COMPLETE -> ArrayList(Arrays.asList("ORDER_BILL_ACCEPTED", "ORDER_FINISTHED"))
            STATUS_MANAGER_CANCEL -> ArrayList(Arrays.asList("ORDER_CANCELED"))
        // 用户
            STATUS_USER_GOING -> ArrayList(Arrays.asList("ORDER_SUBMITTED", "ORDER_ACCEPTED", "ORDER_CAR_PAYED",
                    "ORDER_ILLEGAL_PAYED", "ORDER_EXECUTING", "ORDER_BILL_APPLIED", "ORDER_BILL_ACCEPTED"))
            else -> ArrayList(Arrays.asList("ORDER_FINISTHED", "ORDER_CANCELED"))
        }
    }


    private fun getOrders() {
        if (activity == null)
            return
        val presenter = OrderPresenterImpl(activity, OrdersResult::class.java)
        presenter.addListener(mOrdersModel)
        val request = OrdersRequest()
        request.uid = SharedPreferencesUtil.getInstance().uid
        LogUtil.e("liuyuzhuo page $page")
        request.page = page
        request.page_size = PAGE_SIZE
        request.status = 4
        request.status_list = statusList
        if (SharedPreferencesUtil.getInstance().role == 1 && sortFactor != OrderListSortView.DEFAULT) {
            request.sidx = sidx
            request.sord = sord
        }
        presenter.getOrderList(request)
    }

    private val mOrdersModel = object : GeneralViewInterface {
        override fun getCodeBeanFail(errorCode: Int) {
            refreshLayout.refreshIsEnd()
            refreshLayout.loadMoreIsEnd()
//            emptyLayout(true, NET_ERROR)
        }

        override fun getCodeObjectSuccess(bean: Any?) {
            refreshLayout.refreshIsEnd()
            refreshLayout.loadMoreIsEnd()
            val result = bean as OrdersResult
            LogUtil.e("orderType = $orderType")
            when (result.code) {
                CarEntity.SUCC -> {
                    // 第一页，清除数据，footer
                    if (page == 1) {
                        mOrders?.clear()
                        deleteFooter()
                    }
                    mOrders?.addAll(result.orders ?: ArrayList())
                    if (result.orders?.size ?: 0 < PAGE_SIZE) {
                        // 最后的数据
                        if (result.orders?.size ?: 0 > 0) {
                            addFooter()
                        } else {
                            // 个数为0， 如果是第一页，空页面；不是第一页，添加footer
                            if (page > 1) addFooter()
                        }
                    }
                    mAdapter?.list = mOrders
                    mAdapter?.notifyDataSetChanged()
                    page++
                    LogUtil.e("mAdapter?.notifyDataSetChanged() -----------------------------------------------------")
                }
                else -> showToast(result.msg)
            }
            if (mAdapter?.itemCount == 0) emptyLayout(true, NO_ORDERS) else emptyLayout(false, NO_ORDERS)
        }
    }

    private fun deleteFooter() {
        mAdapter?.addFooter(false)
        isFooterAdded = false
    }

    private fun addFooter() {
        if (isFooterAdded)
            return
        mAdapter?.addFooter(true)
        val footer = OrdersBean()
        mOrders?.add(footer)
        isFooterAdded = true
    }

    private fun emptyLayout(b: Boolean, i: Int) {
        refreshLayout.visibility = if (b) View.GONE else View.VISIBLE
        orderEmpty.visibility = if (b) View.VISIBLE else View.GONE
        when (i) {
            NET_ERROR -> {
                orderEmpty.setEmptyData(R.mipmap.ic_no_network, "", ConstantsUtil.NO_NETWORK, "")
                orderEmpty.setNegativeButton("重新加载", View.OnClickListener { getOrders() })
            }
            NO_ORDERS -> {
                orderEmpty.setEmptyData(R.mipmap.ic_no_orders, "", "您还没有订单哦~", "")
            }
        }
    }

    fun onPostOrders() {
        page = 1
        getOrders()
    }

    fun onSort(type: Int?, b: Boolean) {
        sortFactor = type ?: 0
        sord = if (b) OrderListSortView.ASC else OrderListSortView.DESC
        sidx = when (sortFactor) {
            OrderListSortView.TAKE_CAR -> OrderListSortView.START_TIME
            OrderListSortView.RETURN_CAR -> OrderListSortView.END_TIME
            else -> ""
        }
    }

    fun onItemViewClick(l: OrderItemViewListener) {
        this.listener = l
    }

    interface OrderItemViewListener {
        fun onItemClick(order: OrdersBean?)
        fun onSmallOne(order: OrdersBean?, position: Int)
        fun onSmallTwo(order: OrdersBean?, position: Int)
        fun onBigThree(order: OrdersBean?, position: Int)
    }

    @Suppress("MayBeConstant")
    companion object {
        // 车管
        val STATUS_MANAGER_HANDOVER = 1
        val STATUS_MANAGER_GOING = 2
        val STATUS_MANAGER_SETTLED = 3
        val STATUS_MANAGER_COMPLETE = 4
        val STATUS_MANAGER_CANCEL = 5
        // 用户
        val STATUS_USER_GOING = 6
        val STATUS_USER_HISTORY = 7

        private val PAGE_SIZE = 10
        private val NET_ERROR = 1
        private val NO_ORDERS = 2
    }
}