package com.jiaoyinbrother.monkeyking.view

import android.content.Context
import android.graphics.Canvas
import android.graphics.Paint
import android.support.v7.widget.LinearLayoutManager
import android.support.v7.widget.RecyclerView
import android.util.Log
import android.util.TypedValue
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.TextView

import com.jiaoyinbrother.monkeyking.R
import com.jybrother.sineo.library.bean.SiteBean2
import com.jybrother.sineo.library.listeners.SiteCheckListener
import com.jybrother.sineo.library.util.LogUtil

import java.util.ArrayList

/**
 * The Author : Soda
 * Created on 2017/12/21.
 */
class ItemDecoration(private val context: Context, datas: ArrayList<SiteBean2>) : RecyclerView.ItemDecoration() {
    private val mTitleHeight: Int
    private var mDatas = ArrayList<SiteBean2>()
    private val mInflater: LayoutInflater//    private SiteCheckListener mCheckListener;
    private var mCheckListener: SiteCheckListener? = null

    fun setCheckListener(listener: SiteCheckListener?) {
        mCheckListener = listener
    }

    init {
        this.mDatas = datas
        val paint = Paint()
        mTitleHeight = TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 30f, context.resources.displayMetrics).toInt()
        val titleFontSize = TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_SP, 16f, context.resources.displayMetrics).toInt()
        paint.textSize = titleFontSize.toFloat()
        paint.isAntiAlias = true
        mInflater = LayoutInflater.from(context)
    }


    fun setData(data: ArrayList<SiteBean2>): ItemDecoration {
        this.mDatas = data
        return this
    }


    override fun onDrawOver(canvas: Canvas, parent: RecyclerView, state: RecyclerView.State?) {
        try {
            val pos = (parent.layoutManager as LinearLayoutManager).findFirstVisibleItemPosition()
            Log.e("pos--->", pos.toString())
            val tag = mDatas[pos].key_index
            val child = parent.findViewHolderForLayoutPosition(pos).itemView
            var isTranslate = false//canvas是否平移的标志
            if (mDatas[pos].key_index != mDatas[pos + 1].key_index) {
                Log.e("isTranslate--->", isTranslate.toString() + "")
                val i = child.height + child.top
                Log.d("i---->", i.toString())
                if (mDatas[pos].itemType != SiteBean2.TYPE_TITLE) {
                    //body 才平移
                    if (child.height + child.top < mTitleHeight) {
                        canvas.save()
                        isTranslate = true
                        val height = child.height + child.top - mTitleHeight
                        canvas.translate(0f, height.toFloat())
                    }
                }
            }
            drawHeader(parent, pos, canvas)
            if (isTranslate) {
                canvas.restore()
            }
            Log.d("tag--->", tag.toString() + "VS" + currentTag)
            if (tag != currentTag) {
                currentTag = tag
                mCheckListener?.check(tag, false)
            }
        } catch (e: Exception) {
            LogUtil.e("Exception -------------------------")

        }

    }

    /**
     * @param parent
     * @param pos
     */
    private fun drawHeader(parent: RecyclerView, pos: Int, canvas: Canvas) {
        val topTitleView = mInflater.inflate(R.layout.item_site_title, parent, false)
        val tvTitle = topTitleView.findViewById(R.id.siteTitle) as TextView
        tvTitle.text = mDatas[pos].area
        //绘制title开始
        val toDrawWidthSpec: Int//用于测量的widthMeasureSpec
        val toDrawHeightSpec: Int//用于测量的heightMeasureSpec
        var lp: RecyclerView.LayoutParams? = topTitleView.layoutParams as RecyclerView.LayoutParams
        if (lp == null) {
            lp = RecyclerView.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT)//这里是根据复杂布局layout的width height，new一个Lp
            topTitleView.layoutParams = lp
        }
        topTitleView.layoutParams = lp
        if (lp.width == ViewGroup.LayoutParams.MATCH_PARENT) {
            //如果是MATCH_PARENT，则用父控件能分配的最大宽度和EXACTLY构建MeasureSpec
            toDrawWidthSpec = View.MeasureSpec.makeMeasureSpec(parent.width - parent.paddingLeft - parent.paddingRight, View.MeasureSpec.EXACTLY)
        } else if (lp.width == ViewGroup.LayoutParams.WRAP_CONTENT) {
            //如果是WRAP_CONTENT，则用父控件能分配的最大宽度和AT_MOST构建MeasureSpec
            toDrawWidthSpec = View.MeasureSpec.makeMeasureSpec(parent.width - parent.paddingLeft - parent.paddingRight, View.MeasureSpec.AT_MOST)
        } else {
            //否则则是具体的宽度数值，则用这个宽度和EXACTLY构建MeasureSpec
            toDrawWidthSpec = View.MeasureSpec.makeMeasureSpec(lp.width, View.MeasureSpec.EXACTLY)
        }
        //高度同理
        if (lp.height == ViewGroup.LayoutParams.MATCH_PARENT) {
            toDrawHeightSpec = View.MeasureSpec.makeMeasureSpec(parent.height - parent.paddingTop - parent.paddingBottom, View.MeasureSpec.EXACTLY)
        } else if (lp.height == ViewGroup.LayoutParams.WRAP_CONTENT) {
            toDrawHeightSpec = View.MeasureSpec.makeMeasureSpec(parent.height - parent.paddingTop - parent.paddingBottom, View.MeasureSpec.AT_MOST)
        } else {
            toDrawHeightSpec = View.MeasureSpec.makeMeasureSpec(mTitleHeight, View.MeasureSpec.EXACTLY)
        }
        //依次调用 measure,layout,draw方法，将复杂头部显示在屏幕上
        topTitleView.measure(toDrawWidthSpec, toDrawHeightSpec)
        topTitleView.layout(parent.paddingLeft, parent.paddingTop, parent.paddingLeft + topTitleView.measuredWidth, parent.paddingTop + topTitleView.measuredHeight)
        topTitleView.draw(canvas)//Canvas默认在视图顶部，无需平移，直接绘制
        //绘制title结束
    }

    companion object {
        var currentTag: Int? = 0//标记当前左侧选中的position，因为有可能选中的item，右侧不能置顶，所以强制替换掉当前的tag

        fun setCurrentTag(currentTag: Int) {
            ItemDecoration.currentTag = currentTag
        }
    }
}
