package com.jiaoyinbrother.monkeyking.view

import android.content.Context
import android.content.Intent
import android.support.v4.view.PagerAdapter
import android.support.v4.view.ViewPager
import android.support.v4.view.ViewPager.OnPageChangeListener
import android.text.TextUtils
import android.util.AttributeSet
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.*
import android.widget.AdapterView.OnItemClickListener
import com.facebook.drawee.view.SimpleDraweeView
import com.jiaoyinbrother.monkeyking.CarEntity
import com.jiaoyinbrother.monkeyking.Constants
import com.jiaoyinbrother.monkeyking.R
import com.jiaoyinbrother.monkeyking.activity.GeneralWebViewActivity
import com.jiaoyinbrother.monkeyking.activity.MonkeyMainActivity
import com.jiaoyinbrother.monkeyking.activity.SiteDetailActivity
import com.jiaoyinbrother.monkeyking.activity.SiteListActivity
import com.jiaoyinbrother.monkeyking.view.banner.BannerViewPager
import com.jybrother.sineo.library.bean.FeaturesItem
import com.jybrother.sineo.library.bean.WebViewConfigEntity
import com.jybrother.sineo.library.util.ConstantsUtil
import com.jybrother.sineo.library.util.LogUtil
import com.jybrother.sineo.library.util.StatisticsUtil
import java.util.*

class TagThemeLayout : LinearLayout {

    private lateinit var content_vp: BannerViewPager
    private lateinit var dots_ll: LinearLayout
    private val viewLists = ArrayList<View>()
    private val mContext: Context

    constructor(context: Context) : super(context) {
        mContext = context
        init()
    }

    constructor(context: Context, attrs: AttributeSet) : super(context, attrs) {
        mContext = context
        init()
    }

    private fun init() {
        val layout = View.inflate(context, R.layout.tag_theme_layout, null)
        content_vp = layout
                .findViewById(R.id.tag_theme_layout_content_vp) as BannerViewPager
        dots_ll = layout
                .findViewById(R.id.tag_theme_layout_dots_ll) as LinearLayout
        addView(layout)

    }

    fun addData(list: List<FeaturesItem>) {
        viewLists.clear()

        val page_num: Int
        if (list.size % 4 == 0) {
            page_num = list.size / 4
        } else {
            page_num = list.size / 4 + 1
        }
        var page_index = 0
        var page_size: Int
        for (i in 0 until page_num) {
            val view = View.inflate(mContext, R.layout.tag_theme_pager_item, null)
            val gv = view.findViewById(R.id.tag_theme_pager_item_gv) as GridView
            val newList = ArrayList<FeaturesItem>()

            if (i == page_num - 1 && list.size % 4 != 0) {
                page_size = list.size % 4
            } else {
                page_size = 4
            }
            for (j in page_index until page_index + page_size) {
                newList.add(list[j])
            }
            page_index += 4
            gv.adapter = GridViewAdapter(newList)
            gv.onItemClickListener = GridViewItemClickListener()
            viewLists.add(view)
        }
        content_vp.adapter = MyAdapter()
        content_vp.setOnPageChangeListener(MyPageChangeListener())
        addDots(page_num)
    }

    private fun addDots(page_num: Int) {
        dots_ll.removeAllViews()
        if (page_num <= 1) {
            dots_ll.visibility = View.GONE
            return
        }
        val lyp = LinearLayout.LayoutParams(16, 16)
        lyp.setMargins(0, 0, 10, 0)
        for (i in 0 until page_num) {
            val imageView = ImageView(mContext)
            imageView.layoutParams = lyp
            imageView.setBackgroundResource(R.drawable.selector_dot)
            if (i == 0) {
                imageView.isEnabled = false
            }
            dots_ll.addView(imageView)
        }
    }

    internal inner class GridViewItemClickListener : OnItemClickListener {

        override fun onItemClick(parent: AdapterView<*>, view: View, position: Int,
                                 id: Long) {

            val featureItem = parent.getItemAtPosition(position) as FeaturesItem

            val title = featureItem.title
            val intent: Intent

            var icon_id = position.toString() + 1

            StatisticsUtil.onEvent(mContext, ConstantsUtil.HOME_ICON_+ icon_id, "TO DO")
            LogUtil.e("ConstantsUtil.HOME_ICON_+icon_id =" + ConstantsUtil.HOME_ICON_+icon_id);

            if (!TextUtils.isEmpty(title) && title == "网点") {
                intent = Intent(mContext, SiteListActivity::class.java)
                intent.putExtra("SiteDetailFromType", SiteDetailActivity.TYPE_FROM_MAIN_SITES)
                intent.putExtra(Constants.ENTRY_TYPE, SiteListActivity.TYPE_SITE_LIST)
                (mContext as MonkeyMainActivity).startActivityForResult(intent, Constants.MAIN_RESULT)
                StatisticsUtil.onEvent(mContext, ConstantsUtil.HOME_SITE, "TO DO")
            } else {
                val webViewConfigEntity = WebViewConfigEntity()
                webViewConfigEntity.title = title
                webViewConfigEntity.url = featureItem.url
                webViewConfigEntity.imageUrl = featureItem.pic
                webViewConfigEntity.description = featureItem.description
                webViewConfigEntity.isNeedProgressBar = true
                webViewConfigEntity.isNeedShareButton = true

                intent = Intent(mContext, GeneralWebViewActivity::class.java)
                intent.putExtra(CarEntity.FEATURE_ITEM, webViewConfigEntity)
                mContext.startActivity(intent)
            }


        }

    }

    internal inner class MyAdapter : PagerAdapter() {

        override fun getCount(): Int { // 获得size

            return viewLists.size
        }

        override fun isViewFromObject(arg0: View, arg1: Any): Boolean {

            return arg0 === arg1
        }

        override fun destroyItem(view: View?, position: Int, `object`: Any?) // 销毁Item
        {
            try {
                (view as ViewPager).removeView(viewLists[position])
            } catch (e: Exception) {
            }

        }

        override fun instantiateItem(view: View?, position: Int) // 实例化Item
                : Any {
            (view as ViewPager).addView(viewLists[position], 0)
            return viewLists[position]
        }

    }

    internal inner class MyPageChangeListener : OnPageChangeListener {

        override fun onPageScrollStateChanged(arg0: Int) {


        }

        override fun onPageScrolled(arg0: Int, arg1: Float, arg2: Int) {


        }

        override fun onPageSelected(arg0: Int) {

            for (i in 0 until dots_ll.childCount) {
                val imageView = dots_ll.getChildAt(i) as ImageView
                imageView.isEnabled = i != arg0
            }
        }

    }

    internal inner class GridViewAdapter(private val list: List<FeaturesItem>) : BaseAdapter() {

        override fun getCount(): Int {

            return list.size
        }

        override fun getItem(position: Int): Any {

            return list[position]
        }

        override fun getItemId(position: Int): Long {

            return position.toLong()
        }

        override fun getView(position: Int, convertView: View?, parent: ViewGroup): View? {
            var convertView = convertView

            val holder: ViewHolder
            if (convertView == null) {
                holder = ViewHolder()
                convertView = LayoutInflater.from(mContext).inflate(
                        R.layout.tag_theme_pager_item_gv_item, null)
                holder.img_iv = convertView
                        .findViewById(R.id.tag_theme_pager_item_gv_item_img_iv) as SimpleDraweeView
                holder.name_tv = convertView
                        .findViewById(R.id.tag_theme_pager_item_gv_item_name_tv) as TextView

                convertView.tag = holder
            } else {
                holder = convertView.tag as ViewHolder
            }

            holder.img_iv?.setImageURI(list[position].pic)
            holder.name_tv?.text = list[position].title
            return convertView
        }

    }

    internal inner class ViewHolder {
        var img_iv: SimpleDraweeView? = null
        var name_tv: TextView? = null
    }
}
